#!/usr/bin/perl
use strict;
use Getopt::Long;
use File::Copy;
use lib "/home/p53/classifier_v2.2/scripts";
use ArffUtil;

#Define the script file
my $FEATURE_SCRIPT="nice /home/p53/classifier_v2.2/scripts/runMI.bash";
my $DEF2ARFF="nice /home/p53/classifier_v2.2/scripts/def2arff.bash";
my $ARFFABUT="nice java -Xmx300MB ArffAbut";
my $FILTERFEATURES="nice java -Xmx300MB FilterFeatures";
#my $FILTERFEATURES="nice /home/p53/software/java/FilterFeatures";
my $REMOVECLASS="nice java -Xmx300MB RemoveActive";

#Declare the command line parseing
my $opt_1d; #Include the 1d.arff
my $opt_s1d; #include the s1d.arff
my $opt_2d; #Include the 2d.arff
my $opt_3d; #Include the 3d.arff
my $opt_4d; #Include the 4d.arff
my $opt_baseArff; #Default K2.S
my $opt_mut; #Extract this single mutant from the .arff file
my $opt_2dsize; #The number of 2d features to extract
my $opt_help; #Print out the help
my $opt_arffDir; #<default> The dir where the arff files are kept
my $opt_defDir; #<default> The dir where the def files are kept.
my $opt_outFile; #<default> The name of the output file
my $opt_addClass; #Add the class to the final .arff file
my $opt_load2d; #Load the 2d .arff file
my $opt_save2d; #Save the 2d feature select .arff to file
my $opt_saveTemp; #Don't delete the temporary directory
my $opt_saveFeatures; #Save the .features file from the features selection
my $opt_loadFeatures; #Load a pregenerated featuers file
my $opt_outDir; #The directory to put the outFile in

#Variables to use
my $startDir=$ENV{'PWD'};
my @dimList;
my $startTime; #Start time in seconds
my $usedTime; #Total Time elapsed

GetOptions( "1d" => \$opt_1d,
            "s1d" => \$opt_s1d,
            "2d" => \$opt_2d,
            "3d" => \$opt_3d,
            "4d" => \$opt_4d,
            "baseArff=s" => \$opt_baseArff,
            "mut=s" => \$opt_mut,
            "2dsize=i" => \$opt_2dsize,
            "help" => \$opt_help,
            "arffDir=s" => \$opt_arffDir,
            "defDir=s" => \$opt_defDir,
            "outFile=s" => \$opt_outFile,
            "addClass" => \$opt_addClass,
            "load2d=s" => \$opt_load2d,
            "save2d=s" => \$opt_save2d,
            "saveTemp" => \$opt_saveTemp,
            "saveFeatures=s" => \$opt_saveFeatures,
            "loadFeatures=s" => \$opt_loadFeatures,
            "outDir=s" => \$opt_outDir );
          
if ($opt_1d){
  print "1d: $opt_1d\n"; 
  push (@dimList, "1d");
}
if ($opt_s1d){
  push (@dimList, "s1d");
}
if ($opt_2d){
  push (@dimList, "2d");
}
if ($opt_3d){
  push (@dimList, "3d");
}
if ($opt_4d){
  push (@dimList, "4d");
}

my $arffDir;
if ($opt_arffDir){
  my $baseDir = $ENV{'PWD'};
  chdir $opt_arffDir;
  $arffDir=$opt_arffDir;
  #$arffDir=$ENV{'PWD'};
  chdir $baseDir;
} else {
  $arffDir="/home/p53/classifier_v2.2/rawArff/";
}

my $defDir;
if ($opt_defDir){
  my $baseDir = $ENV{'PWD'};
  chdir $opt_defDir;
  $defDir=$opt_defDir;
  chdir $baseDir;
} else {
  $defDir="/home/p53/classifier_v2.2/def/";
}

my $baseArff;
if ($opt_baseArff){
  $baseArff=$opt_baseArff;
} else {
  $baseArff="K2.S";
}

my $outFile;
if ($opt_outFile){
  $outFile=$opt_outFile;
} else {
  $outFile="composite.arff";
}

my $outDir;
if ($opt_outDir){
  $outDir=$opt_outDir;
} else {
  $outDir=$ENV{'PWD'};
}

if ($opt_help){
  print "Create a composite .arff file from the given parameters\n"; 
  print "--1d \tInclude the 1d.arff\n";
  print "--s1d \tInclude the s1d.arff\n";
  print "--2d \tInclude the 2d.arff\n";
  print "--3d \tInclude the 3d.arff\n";
  print "--4d \tInclude the 4d.arff\n";
  print "--baseArff <prefix> \n\t<default=K2.S>\n";
  print "--mut <mutname> \tExtract this single mutant from the .arff file\n";
  print "--2dsize <size> \tThe number of 2d features to extract\n";
  print "--help \tPrint out the help\n";
  print "--arffDir <dir> \tThe dir where the arff files are kept\n\t<default=$arffDir>\n";
  print "--defDir <dir> \tThe dir where the def files are kept.\n\t<default=$defDir>\n";
  print "--outFile <file> \tThe name of the output file\n\t<default=$outFile>\n";
  print "--addClass \tAdd the class to the final .arff file.  Uses .def file\n";
  print "--load2d <file> \tLoad the 2d .arff from <file>\n";
  print "--save2d <file> \tSave the 2d feature select .arff to file\n";
  print "--saveTemp \tDon't delete the temporary directory\n";
  print "--saveFeatures <file> \tSave the .features file from the features selection\n";
  print "--loadFeatures <file> \tLoad a pregenerated features file\n";
  print "--outDir <dir> \tThe directory where outFile is saved.\n\t<default= the starting directory>\n";
  die;

}

@dimList = sort @dimList;
#print "@dimStack \n";

#Begin copying the correct .arff file(s) to stick together
mkdir "temp_xv";
chdir "temp_xv";

foreach ( @dimList ){
  my $dim=$_;
  print "foreach $_\n";
  print("$dim: copy($arffDir/$baseArff.$dim.arff,$baseArff.$dim.arff)\n");
  #copy($arffDir.$baseArff.".".$dim.".arff",$baseArff.".".$dim.".arff");
  system ("cp $arffDir/$baseArff.$dim.arff $baseArff.$dim.arff");
  if ($opt_mut){
    system("echo $opt_mut > $opt_mut.list");
    system("java PreserveListFromArff $baseArff.$dim.arff $opt_mut.list temp.arff");
    rename("temp.arff","$baseArff.$dim.arff");
  }
}

#Filter out the 2d features
if ($opt_2dsize){
  my $dim="2d";
  my $inName="$baseArff.$dim.arff";
  my $outName="noActivity.best.$dim.class.arff";
  my $numFeatures=3000;
  my $defName="$defDir/$baseArff.def";

  if ($opt_load2d) {
    chdir ("..");
    copy("$opt_load2d","./temp_xv/$baseArff.$dim.arff");
    chdir ("temp_xv");
  } else {
    if($opt_loadFeatures){
      chdir ("..");
      copy ("$opt_loadFeatures","./temp_xv/$dim.features");
      chdir ("./temp_xv");
      ArffUtil->genBestFeatures($inName,$outName,$numFeatures,$defName,"$dim.features");
    } else {
      ArffUtil->genBestFeatures($inName,$outName,$numFeatures,$defName);
      if ($opt_saveFeatures){
        chdir ("..");
        copy("./temp_xv/$outName.features","$opt_saveFeatures");
        chdir ("./temp_xv");
      }
    }
    rename("$baseArff.$dim.arff","unFiltered.$baseArff.$dim.arff");
    rename("$outName","$baseArff.$dim.arff");
    if($opt_save2d){
      chdir ("..");
      copy("./temp_xv/$baseArff.$dim.arff","$opt_save2d");
      chdir ("temp_xv");
    }
  }
}

#Append all files together
print("Append the files\n");
my $isFirst=0;
foreach ( @dimList ){
  my $dim=$_;
  if ($isFirst == 0){
    #copy("$baseArff.$dim.arff","$outFile");
    system("cp $baseArff.$dim.arff $outFile");
    $isFirst=1;
  } else {
    print("system($ARFFABUT $outFile $baseArff.$dim.arff > temp.arff)\n");
    system("$ARFFABUT $outFile $baseArff.$dim.arff > temp.arff");
    copy("temp.arff","$outFile");
  }
}

#Add the class
if ($opt_addClass){
  system("$DEF2ARFF $defDir/$baseArff.def > $baseArff.class.arff");
  system("$ARFFABUT $outFile $baseArff.class.arff > temp.$outFile");
  unlink("$baseArff.class.arff");
  rename("temp.$outFile","$outFile");
}

print("copy($outFile,$outDir/$outFile)\n");
#copy("$outFile","$outDir/$outFile");
system("cp $outFile $outDir/$outFile");

chdir("..");
if (! $opt_saveTemp){
  system("rm -r temp_xv");
}

