#!/usr/bin/perl
package ArffUtil;

use strict;
use warnings;
use Getopt::Long;
use File::Copy;

#A single script to select features from a .arff files

#Variables to use
my $startTime; #Start time in seconds
my $usedTime; #Total Time elapsed

#Define the script file
my $FEATURE_SCRIPT="nice /home/p53/classifier_v3.0/scripts/runMI.bash";
my $DEF2ARFF="nice /home/p53/classifier_v3.0/scripts/def2arff.bash";
my $ARFFABUT="nice java -Xmx300MB ArffAbut";
my $FILTERFEATURES="nice java -Xmx300MB FilterFeatures";
my $REMOVECLASS="nice java -Xmx300MB RemoveActive";

sub testing {
  print "Test = Success\n";
}

#genBestFeature(inName, outName, numFeatures, defName<opt>);
sub genBestFeatures($$$$){
  my($fcnName,$inName, $outName, $numFeatures, $defName, $loadFeatures) = @_;

  print("$fcnName->genBestFeatures: ");
  if ($inName){
    print("inName=$inName, ");
  }

  if ($outName){
    print("outName=$outName, ");
  }

  if ($numFeatures){
    print("numFeatures=$numFeatures, ");
  }

  if ($defName){
    print("defName=$defName, ");
  } 

  if ($loadFeatures){
    print("loadFeatures=$loadFeatures, ");
  }
  print("\n\n");


  #Begin the program
  print("\tFilter $numFeatures features from $inName\n");

  if ($defName){
    $startTime=time;
    
    #my $curDir=$ENV{'PWD'};
    #print("$curDir\n");
    #print("$DEF2ARFF $defName > .class.arff\n");
    system("$DEF2ARFF $defName > .class.arff");
    $usedTime=time - $startTime;
    print "Def2Arff time = $usedTime s\n";

    $startTime=time;
    system("$ARFFABUT $inName .class.arff > .in.arff");
    $usedTime=time - $startTime;
    print "ArffAbut time = $usedTime s\n";

    unlink(".class.arff");
  } else {
    copy($inName,".in.arff");
  }

  if ($loadFeatures){
    copy("$loadFeatures",".$numFeatures.features");
  } else {
    $startTime=time;
    system("$FEATURE_SCRIPT .in.arff $numFeatures > .$numFeatures.features");
    $usedTime=time - $startTime;
    print "runMi time = $usedTime s\n";
  }

  $startTime=time;
  system("$FILTERFEATURES .$numFeatures.features .in.arff 1 > /dev/null");
  $usedTime=time - $startTime;
  print "FilterFeatures time = $usedTime s\n";
  unlink("features.arff");
  rename(".$numFeatures.features","$outName.features");
  unlink(".in.arff");

  $startTime=time;
  system("$REMOVECLASS best..in.arff > /dev/null");
  $usedTime=time - $startTime;
  print "RemoveActive time = $usedTime s\n";
  unlink("best..in.arff");

  rename("noActivity.best..in.arff",$outName);
  return 1;
}

#Return the data line for the given mutant name and dimension (1d,2d,3d,4d,s1d)
sub getDataLine($$$){
  my $mut=shift @_;
  my $dim=shift @_;
  my $newArff=shift @_;

  #my $fileName="/home/p53/classifier_v2.2/rawArff/X2.".$dim.".arff";
  my $fileName=$newArff.".".$dim.".arff";
  open FILE, "<", $fileName or die "Can't open $fileName for reading $!\n";

  my $flag=0;
  while ( my $line = <FILE> )
  {
    if ($flag == 1){
      return $line;
    }
    my $tstline = substr($line,1);
    $tstline=~s/\s+$//;
    #print("$mut\n");
    if ( $mut eq $tstline )
    {
      #print "$line";
      $flag=1;
    }
  }
  close FILE;
}

