import sys
import optparse

class classifierstats:
	def __init__(self,name, tp, fp, fn, tn):
		self.name=name
		self.tp=int(tp)+.5
		self.fp=int(fp)+.5
		self.fn=int(fn)+.5
		self.tn=int(tn)+.5 
	def __str__(self):
		return "#"+ self.name+"\t%g\t%g\t%g\t%g"%(self.tp-.5, self.fp-.5, self.fn-.5, self.tn-.5)
	def update_prob(self, test_result, p_pos):
		tp=self.tp
		fp=self.fp
		tn=self.tn
		fn=self.fn
                #p_pos=(tp+fn)/(tp+fp+tn+fn) #Added v 3.0
		p_neg=1-p_pos
		
		#P(P|D)=P(D|P)P(P)/P(D)
		#P(P)=p_pos
		if not not test_result:
			#P(D|P)=tp/(tp+fn)
			p_d_p=tp/(tp+fn)
			#P(D)
			p_d=tp/(tp+fn)*p_pos+\
			     fp/(fp+tn)*(1-p_pos)
		else:
			#P(D|P)=fn/(tp+fn)	
			p_d_p=fn/(tp+fn)
			#P(D)
			p_d=fn/(tp+fn)*p_pos+\
			    tn/(fp+tn)*(1-p_pos)
		return p_pos*p_d_p/p_d 
	def __repr__(self):
		return '"%s"'%str(self)
		

class bayes_model(dict):
	def read(self, file):
		for cs in stats_iter(file):
			self[cs.name]=cs
	def classify(self,instance, prior=.5):
	#def classify(self,instance, prior=1):
		p=prior
		for  n,r in instance:
			if n in self:
				p=self[n].update_prob(r, p)
		return p

def stats_iter(file):
	for l in file:
		if l[0]=="#":
			l=l[1:].strip().lower().split()
			yield classifierstats(l[0], l[1], l[2], l[3], l[4])

class instance(list):
	def __init__(self):
		self.truth=None
	def add(self, test_name, result):
		r=result
		result=result[0].upper()
		if result=="A" or result=="T" or result=="P":
			result=True
		elif result=="I" or result=="F" or result=="N":
			result=False
		else:
			result=None

		if test_name.upper()=="NAME":
			self.name=r
		elif test_name.upper()=="TRUTH":
			self.truth=result
		elif result==True:
			self.append((test_name, True))
		elif result==False:
			self.append((test_name, False))
		
		

def instance_iter(file):
	for l in file:
		if l[0]=="#":
			header=l[1:].strip().lower().split()
		else:
			i=instance()
			l=l.strip().lower().split()
			for n, v in zip(header, l):
				i.add(n, v)
			yield i	
		

parser=optparse.OptionParser(usage="BAYES [options] confusion_matrices.tab < instances.tab > out.txt " )
parser.add_option(
        "-r", "--result_files", metavar="DIR", action="store", dest="result_dir",
        help="Create 'result' files in DIR.", default=None)
parser.add_option(
        "-t", "--threshold", metavar="P", action="store", dest="threshold", type="float",
        help="Probability threshold at which to classify instance as positive or negative.", default=.5)
#parser.add_option(
#        "-l", "--labels", action="store_true", dest="invert", default=False,
#        help="Invert selection of filter so molecules that would have been dropped are kept and vice versa.")
parser.add_option(
	"-q", "--quiet", action="store_true", dest="quiet", default=False, help="Don't output results to stdout.")

(options, args) = parser.parse_args()

model=bayes_model()
model.read(open(sys.argv[1]))

def translate(true_false, true="active", false="inactive", none="?"):
	if true_false==True:
		return true
	elif true_false==False:
		return false
	else:
		return none

for i in instance_iter(sys.stdin):
	c=model.classify(i)
	if not options.quiet:
		print i.name +"\t"+ str(c)
	if options.result_dir:
		try:
			f=open(options.result_dir + "/" + i.name + ".result", "w")
			print >>f, "PRED:\t%s"%translate(c>options.threshold)
			print >>f, "TRUTH:\t%s"%translate(i.truth)
			print >>f, "P:\t%f"%c
		finally:
			f.close()
	

	
#Version 3.0  Sam Danziger 14:20:00  4/01/05
#    Modified prior to tp+fn/N
