#!/bin/bash
#Resubmit any mutants that failed
#run in the directory that contains the .arff file and the results_xvalid dir

ARFF=$1
WHICHXVAL=$2 #0: standard OECV, 1: fast OECV (default)

if [ -z $WHICHXVAL ]
then
  WHICHXVAL=1
fi

GETIDX="nice /home/p53/classifier_v3.0/scripts/idxFromArff.bash"
QUEUEDIR="/home/p53/queues/"
if [ $WHICHXVAL -eq 0 ]
then
  RUNCLASS="/home/p53/classifier_v2.2/scripts/runclassifier.allNode"
else
  RUNCLASS="/home/p53/classifier_v3.0/scripts/runclassifier.chunk"
fi
allNodes=`cat ${QUEUEDIR}*.* | tr ',\n' ','`
PREDICT="qsub -p -10 /home/p53/classifier_v3.0/scripts/predict.sge"
COMPRESS="bzip2 *.arff"

if [ -a train.arff.bz2 ]
then
  echo "Unzipping bzip2 ed files"
  bunzip2 *.bz2
fi

if [ ! -d results ]
then
  echo "Resubmitting due to missing results"
  $PREDICT test.arff
fi

#First make sure that results_xvalid exists
if [ ! -d results_xvalid ]
then
  echo "Resubmit due to missing results_xvalid"
  $RUNCLASS $ARFF
  exit 0
fi

#Make sure that there are the correct number of .result files
numMut=`cat $ARFF | grep % | wc | awk '{ print $1 }'`
numRes=`/bin/ls ./results_xvalid/*.result | wc | awk '{ print $1 }'`
if [ $numMut -ne $numRes  ]
then
  echo "Resubmit due to file count mismatch"
  echo "numMut = $numMut, numRes = $numRes"
  rm -r "./results_xvalid"
  $RUNCLASS $ARFF
  exit 1
fi

#Make sure each .result file is valid
cd results_xvalid
  rm "*.result" 2> /dev/null #this is an known error file
  broken=`grep null *.result | awk -F. '{ print $1 }'`
  broken2=`find . -size 0 -print | sed -e 's/\.\///' | sed -e 's/.result//'`
cd ..

resubFlag=0
if [ ! -z "$broken"  ]
then
  $RUNCLASS $ARFF
  resubFlag=1
fi

if [ ! -z "$broken2" ]
then
  if [ $resubFlag -eq 0 ]
  then
    $RUNCLASS $ARFF
  fi 
fi

exit 2


#Version 3.0  Sam Danziger 3/11/05 15:11:55
#  Modified to work in version 3.0
#
#Version 3.1  Sam Danziger 6/21/05 16:10:17
#  Will also check to see if the results directory is there and 
#  If anything is bzip2 ed
