#!/bin/bash
#Run a fully crossvalidated composite prediction for the classifier.
# Should even work with aggregate (1d.2d) classifiers
trainPrefix=$1 #Ex K3wm
dataDir=$2 #Ex: /home/p53/classifier_v3.0 has "rawArff" and "def" subs  
num2dFeat=$3 #Ex: 400 for broad compressed, 3000 for 9778 features
dims=$4 #Default: 1d 2d 3d 4d s1d
startCount=$5 #used to resume if I have to restart.  DEFAULT: 1

echo "Starting xvalidateComposite.bash V3.1"

if [ -z $5 ]
then
  startCount=1
  skipCreateFiles=0
else 
  skipCreateFiles=1
fi

XVALID="nice python /home/p53/classifier_v3.0/scripts/python/ARFFxvalid.py --max_mutations=1"
ARFF2DEF="nice java -Xmx300M Arff2Def"
PREDALLMUT="nice /home/p53/classifier_v3.0/scripts/predAllMut.bash"

if [ -z "$4" ]
then
  dims="1d 2d 3d 4d s1d"
fi


numSplits=0
for dim in $dims
do
  numSplits=`$XVALID $dataDir/rawArff/$trainPrefix.$dim.arff`
done

if [ $skipCreateFiles -eq 0 ]
then
  echo "Deleting Old Files"
  if [ -e components ]
  then
    rm -r components
  fi
  mkdir components
  mkdir results
fi

cd components

if [ $skipCreateFiles -eq 0 ]
then
  echo "Creating $numSplits Xvalidations"
  count=$startCount
  while [ $count -le $numSplits ]
  do
    mkdir $count
    mkdir $count/rawArff
    mkdir $count/def
    for dim in $dims
    do
      arff=$dataDir/rawArff/$trainPrefix.$dim.arff
      def=$dataDir/def/$trainPrefix.def
      echo $arff
      $XVALID $arff $count $count > $count/rawArff/train.$dim.arff
      $ARFF2DEF $count/rawArff/train.$dim.arff | awk '{ print $1 }'> $count/def/train.list
      cat $def | grep -i -f $count/def/train.list > $count/def/train.def
      curMut=`cat $count/*.arff | grep % | sed -e 's/%//'`
      mv ./$count/$curMut.arff $count/rawArff/$curMut.$dim.arff
      $ARFF2DEF $count/rawArff/$curMut.$dim.arff | awk '{ print $1 }' > $count/def/$curMut.list
      cat $def | grep -i -f $count/def/$curMut.list > $count/def/$curMut.def
    done
    mv $count $curMut
    count=$(( $count + 1 ))
  done
fi #End skip option

#Submit each split for xvalidations
muts=`/bin/ls -d */ | sed -e 's/\///'`
idx=$startCount
for mut in $muts
do
  echo "Submitting $mut"
  cd $mut
  curDir=`pwd`
  m=$(( $idx % 15 ))
  if [ $m -eq 0 ]
  then
    echo "Waiting for $idx"
    $PREDALLMUT train $mut $curDir 1 $num2dFeat > $mut.out 
  else
    echo "Not waiting for $idx"
    $PREDALLMUT train $mut $curDir 1 $num2dFeat > $mut.out &
  fi
  cd ..
  idx=$(( $idx + 1 ))
done 2> out.err
cd ..

#Version 3.1  Sam Danziger 4/01/05 22:33
#  Added resume option
#Version 3.0  Sam Danziger 3/14/05 14:06:22
#  Rewritten to work in classifier v3.0 framework
