import java.util.*;
import java.io.*;

/**
 * 
 * @author sad1225
 *
 * Add the active,inactive,other classification to a 2d .arff file
 * Base these predictions on a value in a .def file
 * *NOTE: if a value other than active/inactive is present, will
 * pass this on in the @data section, but not in the @attribute section
 */
class AddActivity{
	ArrayList mutNames;
	ArrayList mutActivity;
	
	public AddActivity(String defFileName, String inArff, String outArff){
		mutNames = new ArrayList();
		mutActivity = new ArrayList();
		
		readDef(defFileName);
		processArff(inArff,outArff);
	}
	
	/**
	 * Read in inArff, and add the activity information 
	 * @param inArff
	 * @param outArff
	 */
	private void processArff(String inArff, String outArff){
		try{
			FileReader inFile = new FileReader(inArff);
			BufferedReader arffIn = new BufferedReader(inFile);
			
			FileWriter outFile = new FileWriter(outArff);
			PrintWriter arffOut = new PrintWriter(outFile);

			String line = arffIn.readLine();
			while(!((line.toLowerCase()).startsWith("@data"))){
				arffOut.println(line);
				line = arffIn.readLine();
			}
			
			arffOut.println("@attribute class {active,inactive}");
			arffOut.println(line);
			
			line = arffIn.readLine();
			while (line !=null){
				String curMut = line.substring(1);				
				System.out.println(curMut);
				
				int idx = mutNames.indexOf(curMut);

                if (idx != -1){ 				
					arffOut.println(line);
                }
				line = arffIn.readLine();
				if (idx != -1){ 				
					String activity = (String)(mutActivity.get(idx));
					arffOut.println(line+","+activity+",");
				}
				line = arffIn.readLine();
			}
			
			arffIn.close();
			arffOut.close();

		} catch (Exception e){
			e.printStackTrace();
		}		
	}
	
	/**
	 * Read in the active/inactive status from a .def file
	 * @param defFileName
	 */
	private void readDef(String defFileName){
		try{
			FileReader inFile = new FileReader(defFileName);
			BufferedReader defReader = new BufferedReader(inFile);
			
			String line = defReader.readLine();
			while(line !=null){
				StringTokenizer tok = new StringTokenizer(line);
				mutNames.add(tok.nextToken());
				mutActivity.add(tok.nextToken());
				line = defReader.readLine();
			}
			defReader.close();
			
		} catch (Exception e){
			e.printStackTrace();
		}		
	}
	public static void main (String args []){
		if (args.length != 3){
			System.out.println("Combine the many .arff files created for 2d");
			System.out.println("\tAddActivity <defFileName>, <inArff>, <outArff>");
			System.out.println("ex: AddActivity aircComplete.list.def best.aircCompleteWeka.arff 2d.best.arff");
		} else {
			AddActivity myAddActivity = new AddActivity(args[0], args[1], args[2]);			
		}
	}	

}