import java.io.*;

/**
 * 
 * @author Sam Danziger
 *
 * Add the "name" field as the first attribute in a .arff files.  Assumes
 * the convention %name in the line before each data line.
 * 
 * Write results to standard out.
 */

class AddNameToArff {
	public AddNameToArff(String arffName){
		try {
			FileReader inFile = new FileReader(arffName);
			BufferedReader in = new BufferedReader(inFile);			

			//Title			
			String line = in.readLine();
			while (!(line.toLowerCase()).startsWith("@attribute")){
				System.out.println(line);
				line = in.readLine();
			}

			//Attributes
			System.out.println("@ATTRIBUTE\tname\tSTRING");
			while (!(line.toLowerCase()).startsWith("@data")){
				System.out.println(line);
				line = in.readLine();
			}

			//@data
			System.out.println(line);
			line = in.readLine();
			
			while (line != null){
				//Name Line
				System.out.println(line);
				String name = line.substring(1);

				//Data line
				line = in.readLine();
				System.out.println(name+","+line);
				line = in.readLine();
			}

			in.close();
		} catch (Exception e){
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	 * Add the "name" field as the first attribute in a .arff files.  Assumes
 	 * the convention %name in the line before each data line.
 	 * 
 	 * Write results to standard out.
	 * AddNameToArff <arffName>
	 * AddNameToArff 3d.arff > named.3d.arff
	 *
	 * @param args
	 */
	public static void main (String args []) {
		if (args.length != 1){
			System.out.println("Add the name field as the first attribute in a .arff files.  Assumes the convention %name in the line before each data line.");
			System.out.println("AddNameToArff <arffName>" );
			System.out.println("AddNameToArff 3d.arff > named.3d.arff" );
		} else {
			String arffName = args[0];
			AddNameToArff myAddNameToArff = new AddNameToArff(arffName);
			//System.out.println("Completed Execution: AddNameToArff "+arffName);
		}
	}

}