import java.io.*;
import java.util.*;

/**
 * 
 * @author Sam Danziger
 *
 * Combine the features of Arff files with the same name and write 
 * to standard out. 
 */
class ArffAbut {
	ArrayList mutNames1;
	ArrayList data1;
	boolean d1HasNames;
	ArrayList mutNames2;
	ArrayList data2;
	boolean d2HasNames;
	/**
	 * Constructor
	 * @param inFile1	  The first .arff Files
	 * @param inFile2	  The second .arff Files
	 */
	public ArffAbut(String inFile1, String inFile2){
		mutNames1 = new ArrayList();
		data1 = new ArrayList();
		mutNames2 = new ArrayList();
		data2 = new ArrayList();

		try{
			FileReader arffInFile = new FileReader(inFile1);
			BufferedReader inArff1 = new BufferedReader(arffInFile);

			FileReader inFile = new FileReader(inFile2);
			BufferedReader inArff2 = new BufferedReader(inFile);

            //Step 0:  If one file is empty, echo the other one
			String arffLine = inArff1.readLine();	
			String arff2Line = inArff2.readLine();  //%<mutName>
			if (arffLine == null){
				if (arff2Line == null){
					return;
				} else {
					while (arff2Line != null){
						System.out.println(arff2Line); 
						arff2Line = inArff2.readLine();
					}
					return;
				}
			}
			
			if (arff2Line == null){
				if (arffLine == null){
					return;
				} else {
					while (arffLine != null){
						System.out.println(arffLine); 
						arffLine = inArff1.readLine();
					}
					return;
				}
			}

			//Step 1:  Copy the header from the first file
			while(!((arffLine.toLowerCase()).startsWith("@data"))){
				System.out.println(arffLine);
				arffLine = inArff1.readLine();
				
				if (arffLine == null) {
					throw new Exception("Never Found @data in First File");
				}
			}

			//Step 2:  Read in the data from the first file
			arffLine = inArff1.readLine();  //%<mutName>
			if ((arffLine.toLowerCase()).startsWith("%")){
				d1HasNames = true;
			} else {
				d1HasNames = false;
			}

			while(arffLine!=null){
				
				if (d1HasNames == true){
					mutNames1.add(arffLine.toLowerCase());
					arffLine = inArff1.readLine();  //data line
				}
				
				if (arffLine != null){
					arffLine = arffLine.trim();
					if ((arffLine.lastIndexOf(",")+1) != arffLine.length()){
						arffLine = arffLine + ",";
					}
					data1.add(arffLine.toLowerCase());
					arffLine = inArff1.readLine();
				}
			}

			//Step 3:  Copy the header from the second file
			while(!((arff2Line.toLowerCase()).startsWith("@data"))){
				if (((arff2Line.toLowerCase()).startsWith("@attribute"))) {
					System.out.println(arff2Line);
				}
				arff2Line = inArff2.readLine();
				if (arff2Line == null) {
					throw new Exception("Never Found @data in Second File");
				}
			}
			System.out.println(arff2Line);

			//Step 4:  Read in the data from the second file
			arff2Line = inArff2.readLine();	
			if ((arff2Line.toLowerCase()).startsWith("%")){
				d2HasNames = true;
			} else {
				d2HasNames = false;
			}

			while(arff2Line!=null){
				if (d2HasNames == true){
					mutNames2.add(arff2Line.toLowerCase());
					arff2Line = inArff2.readLine();
				}
				data2.add(arff2Line.toLowerCase());
				arff2Line = inArff2.readLine();  
			}
			
			//Step 5: Print the matches
			int arff1used=0;
			if (d1HasNames==true && d2HasNames==true){
				for (int i=0; i<mutNames1.size(); i++){
					String curMut = (String)(mutNames1.get(i));
					int idx = mutNames2.indexOf(curMut);
					if (idx > -1){
						arff1used++;
						System.out.println(curMut);
						String d1 = (String)data1.get(i);
						String d2 = (String)data2.get(idx);
						System.out.println(d1+d2);
					}
				}
			} else {
				for (int i=0; i<data1.size(); i++){
					arff1used++;
					String d1 = (String)data1.get(i);
					String d2 = (String)data2.get(i);
					System.out.println(d1+d2);
				}
			}

			//Step 6: Print Results
			System.err.println("Finished abut for "+inFile1+" + "+inFile2);
			System.err.println(arff1used+"/"+data1.size()+" Matched");

			inArff1.close();
			inArff2.close();
		} catch (Exception e){
			e.printStackTrace();
			System.err.println(this.getCurrentDir());
			System.err.println("File 1 : "+inFile1);
			System.err.println("File 2 : "+inFile2);
		}

	}

	/**
	 * Return the current directory in a formatted string.  
	 * Stolen from: Real Gagnon (http://www.rgagnon.com/javadetails/java-0076.html) 
	 * @return
	 */
	public String getCurrentDir() {
      File dir1 = new File (".");
	  File dir2 = new File ("..");
	  String dirName="ERROR: COULD NOT FIND CURRENT DIRECTORY";
	  try {
	    dirName="Current dir : " + dir1.getCanonicalPath() + "\nParent  dir : " + dir2.getCanonicalPath();
	  } catch(Exception e) {
	    e.printStackTrace();
	  }
	  return dirName;
	}
	 
	/**
	 * Provide command line interface
	 * @param args
	 */
	public static void main (String args []){
		if (args.length != 2){
			System.out.println("Combine the features of Arff files with the same mutants and write to standard out. ");
			System.out.println("\tArffAbut <file1> <file2>");
			System.out.println("ex: ArffAbut 1d.arff 2d.arff > 1d.2d.arff");
		} else {
			ArffAbut myArffAbut = new ArffAbut(args[0], args[1]);
		}
	}
}



/**
*Version 3.0  Sam Danziger 3/09/05 10:43:43
*  Modified to work with Classifier Version 3.0.  Add more extensive error checking and messages. 
*/