import java.io.*;
import java.util.*;

/**
 * 
 * @author Sam Danziger
 *
 * Break the .arff file up based on hte first name in each mutant for
 * cross validation
 */
class BreakupArff {
	String curMut;
	ArrayList header; //Store the header information here\
	LinkedHashSet mutantNames;
	
	public BreakupArff(String dirName, String arffName){
		//Read in all of the mutant names
		mutantNames = new LinkedHashSet();
		
		try {
			FileReader inFile = new FileReader(dirName+"/"+arffName);
			BufferedReader in = new BufferedReader(inFile);
			String line = in.readLine();

			while (!(line.toLowerCase()).startsWith("@data")){
				line = in.readLine();				
			}
				
			while (line !=null){
				if (line.startsWith("%")){
					mutantNames.add(get1stMut(getMutName(line)));
				}
				line = in.readLine();
			}				
			in.close();
			
			ArrayList mutNames = new ArrayList(mutantNames);			
			for(int i=0; i<mutNames.size(); i++){
				String mutName = (String)(mutNames.get(i));
				separateMutant(dirName, arffName, mutName);
			}
			
		} catch (Exception e){
			System.out.println(e.getMessage());
			e.printStackTrace();
		}	
	}

	/**
	 * Separate all mutants starting with mut1 from the rest
	 * @param dirName
	 * @param arffName
	 * @param mut1
	 */
	private void separateMutant(String dirName, String arffName, String mut1){
		try {
			String arffDir = dirName+"/intermediate";
			FileReader inFile = new FileReader(dirName+"/"+arffName);
			BufferedReader in = new BufferedReader(inFile);

			readHeader(in); // read in header info, return just after @data
			
			//Open the output file for the not mut1
			PrintWriter notOut = writeHeader(arffDir + "/" + mut1 + ".not.arff");
						
			//Iterate through each line an determine where to write it
			String line = in.readLine();
			int ctr = 0;
			while (line != null){								
				if (sameMut(mut1,line)){
					String mutName = getMutName(line);
					PrintWriter out = writeHeader(arffDir + "/" + mut1 + "." + ctr + "." + mutName + ".arff");
					
					out.println("%"+mutName);

					//Process the data line
					line = in.readLine();
					out.println(line);
					out.close();	
					ctr++;
				} else { //Add the mutant the the not .arff File
					notOut.println(line);
					line = in.readLine();
					notOut.println(line);
				}
				line = in.readLine();
			}
			notOut.close();
		} catch (Exception e){
			System.out.println(e.getMessage());
			e.printStackTrace();
		}	
	}
	
	/**
	*Parse a comment line to get the mutant name 
	* @param line
	*/ 
	private String getMutName (String line){
		StringTokenizer tok = new StringTokenizer(line,"%");
		String newMut = tok.nextToken();
		tok = new StringTokenizer(newMut);
 		newMut = tok.nextToken();
		return newMut;
	}
	
	/**
	 * Get the first mutant in mutName
	 * @param mutName
	 * @return
	 */
	private String get1stMut(String mutName){
		StringTokenizer tok = new StringTokenizer(mutName,"_");
		String mut1 = tok.nextToken();
		return mut1;		
	}

	/**
	*Determine from newline if curmut and newLines start with the same 
	*mutant
	* @param curMut
	* @param newLine
	*/ 
	private boolean sameMut(String curMut, String newLine){
		String newMut = getMutName(newLine);

		if ((newMut.toLowerCase()).startsWith(curMut.toLowerCase())){
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Read the header (non-data) information in and store it
	 * in header.
	 * @param in
	 */
	private void readHeader(BufferedReader in){
		header = new ArrayList();
		try {
			String line = in.readLine();
			while (!((line.toLowerCase()).startsWith("@data"))){
				header.add(line);
				line = in.readLine();
			}
		} catch (Exception e){
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	*Write the header information to fileName and return the PrintWriter
	*Return null if something goes wrong 
	*@param fileName
	*/
	private PrintWriter writeHeader(String fileName){
		PrintWriter output = null;
		try {
 			output = new PrintWriter( new FileWriter(fileName));
			for (int i=0; i<header.size(); i++){
				output.println((String)(header.get(i)));
			}
			output.println("@DATA");
		}catch (Exception e){
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		return output;	
    }			
	
	/**
	 * Combine the results files to make a new .arff file
	 * Result2Arff <dirListName>, <outName>, <startDir -- optional>
	 * Result2Arff dirList composite.arff
	 * 
	 * @param args
	 */
	public static void main (String args []) {
		if (args.length != 2){
			System.out.println("Break up the .arff file" );
			System.out.println("BreakupArff <arffDir>, <arffName>" );
			System.out.println("Ex: BreakupArff ./3d 3d.arff" );
		} else {
			String arffDir = args[0];
			String arffName = args[1];
			BreakupArff myBreakupArff = new BreakupArff(arffDir,arffName);
			System.out.println("Completed Execution: BreakupArff "+arffDir+" "+arffName);
		}

	}
	
}