import java.io.*;
import java.util.*;

/**
 * 
 * @author Sam Danziger
 *
 * Read in a composite.arff files and calculate all correlation results
 * between all classifiers using pearson's coefficient
 */
class CorrelateClassifiers{
	ArrayList lists;
	ArrayList listNames;
	ArrayList mutNames;
	
	public CorrelateClassifiers(String arffName, String predName){
		ReadCompositeArff(arffName);
		ReadPrediction(predName);
		//System.out.println(listNames);
		//System.out.println(lists.get(3));
		
		//ADD THE COMPOSITE RESULTS		
		
		//Calculate the pearson's coefficient for each combo
		for (int i=0; i<listNames.size(); i++){
			for (int j=(i+1); j<listNames.size(); j++){
				System.out.println(listNames.get(i)+" + "+listNames.get(j));
				double r = calcPearson((ArrayList)(lists.get(i)),(ArrayList)(lists.get(j)));
				System.out.println("\tPearson Coefficient = " + r);
			}			
		}
	}
	
	/**
	 * Read in the prediction file to get the composite
	 * predictions
	 * @param predName
	 */
	private void ReadPrediction(String predName){
		listNames.add("composite");
		ArrayList compList = new ArrayList();
		
		//Initialize all values
		int listSize = ((ArrayList)(lists.get(0))).size();
		for (int i=0; i<listSize; i++){
			compList.add(new Double(.5));
		}
		
		try {
			FileReader inFile = new FileReader(predName);
			BufferedReader in = new BufferedReader(inFile);
			
			String line = in.readLine();
			while ((line != null) && (line.length() != 0)){
				StringTokenizer tok = new StringTokenizer(line);
				
				//Read in name
				String path = tok.nextToken();
				int slshIdx = path.lastIndexOf("/");
				String name = path.substring(slshIdx+1);
				int mutIdx = mutNames.indexOf(name.toLowerCase());
				
				//Read in activity
				String activity = tok.nextToken();
				double actValue;
				if ((activity.toLowerCase().equals("active"))){
					actValue = 1;
				} else {
					actValue = 0;
				}

				if (mutIdx > -1) {															
					compList.set(mutIdx,new Double(actValue));				
				}				
				line = in.readLine();
			}
			
		} catch (Exception e){
			e.printStackTrace();
		}
		lists.add(compList);
	}
	
	/**
	 * Read the composite .arff file to fill lists @ listNames
	 * @param arffName
	 */
	private void ReadCompositeArff(String arffName){
		lists = new ArrayList();
		listNames = new ArrayList();
		mutNames = new ArrayList();
		
		try {
			FileReader inFile = new FileReader(arffName);
			BufferedReader in = new BufferedReader(inFile);
			
			String line = in.readLine();
			while(!((line.toLowerCase()).startsWith("@attribute"))){
				line = in.readLine();
			}
			
			while((line.toLowerCase()).startsWith("@attribute")){
				StringTokenizer tok = new StringTokenizer(line);
				String curTok = tok.nextToken();
				curTok = tok.nextToken();
				if (!curTok.equals("truth")){
					StringTokenizer tok2 = new StringTokenizer(curTok,"/");
					String name = tok2.nextToken();
					listNames.add(name);
				}				
				line = in.readLine();
			}
			
			//@data
			line = in.readLine();
			
			//Prepair the arrays for the different lists
			for(int i=0; i<listNames.size(); i++){
				lists.add(new ArrayList());
			}
			
			while(line != null){
				String name = line.substring(1);
				mutNames.add(name.toLowerCase());				
				line = in.readLine(); //Remove comment line
				
				StringTokenizer tok = new StringTokenizer(line,",");
				int idx = 0;
				while (tok.hasMoreTokens()){
					if (idx < listNames.size()){ //if not on the truth
						String curTok = tok.nextToken();
						if ((curTok.toLowerCase().equals("active"))){
							((ArrayList)(lists.get(idx))).add(new Double(1));
						} else if ((curTok.toLowerCase().equals("inactive"))){
							((ArrayList)(lists.get(idx))).add(new Double(0));
						} else {
							((ArrayList)(lists.get(idx))).add(new Double(.5));
						}
					} else {
						break;				
					}
					idx++;
				}
				
				line = in.readLine();
			}
			
		} catch (Exception e){
			e.printStackTrace();
		}

	}
	
	/**
	 * r = (sum(x*y)-sum(x)*sum(y)/n) / 
	 * sqrt((sum(x^2)-sum(x)^2/n)*(sum(y^2)-sum(y)^2/n))
	 * @param x
	 * @param y
	 * @return
	 */
	private double calcPearson(ArrayList x, ArrayList y){
		double num1 = sumVect(multVect(x,y));
		double sumX = sumVect(x);
		double sumY = sumVect(y);
		int n  = x.size();
		double num = num1 - (sumX*sumY/n);
		
		double den1 = sumVect(multVect(x,x)) - (sumX*sumX/n);
		double den2 = sumVect(multVect(y,y)) - (sumY*sumY/n);
		double den = Math.sqrt(den1*den2);
		
		double r = num/den;
		
		if (Double.isNaN(r)){
			return 0;
		} else {
			return r;
		}
	}
	
	/**
	 * Multiply x .* y (element i time element i)
	 * @param x
	 * @param y
	 * @return
	 */
	private ArrayList multVect(ArrayList x, ArrayList y){
		ArrayList results = new ArrayList();
		for (int i=0; i<x.size(); i++){
			double curX = ((Double)(x.get(i))).doubleValue();
			double curY = ((Double)(y.get(i))).doubleValue();
			double xy = curX * curY;
			results.add(new Double(xy));
		}
		return results;
	}
	
	/**
	 * Return the sum of all values in ArrayList
	 * @param x
	 * @return
	 */
	private double sumVect(ArrayList x){
		double sum = 0;
		for (int i=0; i<x.size(); i++){
			double curX = ((Double)(x.get(i))).doubleValue();
			sum = sum+curX;
		}
		return sum;
	}
	
	/**
	 * Using the data encapsulated in the in.arff file used by the composite 
	 * classifier and a predictions file created by parseing the the .result
	 * files created by leave 1 out crossvalidating the composite predictor,
	 * determine the pearson's correlation coefficient between the different
	 * classifiers. 
	 * CorrelateClassifiers <in.arff> <predictions.txt>
	 * 	 Ex: CorrelateResults in.arff predictions
	 * @param args
	 */
	public static void main (String args []) {
		if (args.length != 2){
			System.out.println("Using the data encapsulated in the in.arff file used by the composite classifier and a predictions file created by parseing the the .result files created by leave 1 out crossvalidating the composite predictor, determine the pearson's correlation coefficient between the different classifiers." );
			System.out.println("CorrelateClassifiers <in.arff> <predictions.txt>" );
			System.out.println("Ex: CorrelateResults in.arff predictions" );
		} else {
			CorrelateClassifiers myCC = new CorrelateClassifiers(args[0],args[1]);
		}
		
		/*
		double [] x = {2,4,6,8,10};
		//double [] y = {5,4,3,2,1};
		double [] y = {1,2,3,4,10};
		//double [] y = {3,3,3,3,3};
		
		ArrayList ax = new ArrayList();
		ArrayList ay = new ArrayList();
		for (int i=0;i<x.length;i++){
			ax.add(new Double(x[i]));
			ay.add(new Double(y[i]));
		}

		System.out.println("sum(x) = "+myCCR.sumVect(ax));
		System.out.println("sum(y) = "+myCCR.sumVect(ay));
		System.out.println("x .* y = "+myCCR.multVect(ax,ay));
		System.out.println("r = "+myCCR.calcPearson(ax,ay));
		System.out.println("r2 = "+myCCR.calcPearson(ay,ax));
		*/
	}
}