import java.io.*;
import java.util.*;

class ArffResultFileFilter implements FilenameFilter {
   public boolean accept(File dir, String file) {
	  return (file.toLowerCase()).endsWith(".arffresult");
   }
}

/**
 * 
 * @author sad1225
 *
 * Create one of these for each individual feature.
 */
class FeatureObj implements Comparable{
	private String name;
	private ArrayList values;
	
	public FeatureObj(String _name){
		values = new ArrayList();
		name = _name;
	}
	
	public void addValue(int value){
		values.add(new Integer(value));
	}
	
	/**
	 * Return the average rank of the feature
	 * @return
	 */
	public double getAverage(){
		int sum=0;
		int valSize=values.size();
		for (int i=0; i<valSize; i++){
			Integer tmpVal=(Integer)(values.get(i));
			sum=sum+tmpVal.intValue();
		}
		return ((double)sum) / ((double)valSize);
	}
	
	/**
	 * Return the name field
	 * @return
	 */
	public String getName() {
		return name;
	}

	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Object arg0) {
		FeatureObj compObj=(FeatureObj)arg0;
		
		if (this.getAverage() < compObj.getAverage()){
			return -1;
		} else if (this.getAverage() > compObj.getAverage()) {
		    return 1;	
		} else {		
			return 0;
		}
	}
}

/**
 * The main class
 * @author sad1225
 *
 */
class CountFeatureVotes {
	private ArrayList featureList;

	/**
	 * Read all. result files in dirName and return the top numFeatures features
	 * @param dirName
	 * @param numFeatures
	 */
	public CountFeatureVotes(String dirName, ArrayList numFeatures){
		featureList=new ArrayList();
		File f1 = new File(dirName);
		String [] fileList=f1.list(new ArffResultFileFilter());
		for (int i=0; i < fileList.length; i++){
			String curFile=fileList[i];
			readFile(dirName+"/"+curFile);
		}
		
		Object objList[] = featureList.toArray();
		Arrays.sort(objList);
		
		/*Print out the results*/
		for (int numCtr=0;numCtr<numFeatures.size();numCtr++){
			String sCurNum=(String)(numFeatures.get(numCtr));
			int curNum=(Integer.valueOf(sCurNum)).intValue();
			
			System.out.println("list: "+curNum+" features");
			for (int i=0;i<curNum;i++){
				if (i==curNum){
					break;
				}
				System.out.print(((FeatureObj)(objList[i])).getName()+",");
			}
			System.out.print("\n\n");
		}
	}
	
	/**
	 * Read in the rankings from a file
	 * @param fileName
	 */
	private void readFile(String fileName){
		FileReader inFile;
		try {
			inFile = new FileReader(fileName);
			BufferedReader in = new BufferedReader(inFile);
			
			String line=(in.readLine()).toLowerCase();
			int rank=0;
			while (!line.startsWith("@data") && line!=null){				
				if (line.startsWith("@attribute")){
					rank++;
					StringTokenizer tok = new StringTokenizer(line);
					String curTok=tok.nextToken();
					curTok=tok.nextToken();
					processFeature(curTok,rank);
				}
				line=(in.readLine()).toLowerCase();
			}
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
	}
	
	/**
	 * It the current mutant already in the list add the new rank.
	 * Otherwise add the mutant and store its rank 
	 * @param name
	 * @return
	 */
	private void processFeature(String name, int rank){
		boolean hasMut=false;
		for (int i=0;i<featureList.size();i++){
			FeatureObj curFeat = (FeatureObj)featureList.get(i);
			String curName=curFeat.getName();
			if (name.equalsIgnoreCase(curName)){
				hasMut=true;
				curFeat.addValue(rank);
				return;
			}
		}
		if (hasMut==false){
			FeatureObj newFeature= new FeatureObj(name);
			newFeature.addValue(rank);
			featureList.add(newFeature);
		}
	}
	
	
	
	public static void main (String args []) {
		String dirName=null;
		ArrayList numFeatures=new ArrayList();
		try {
			for (int i=0;i<args.length;i++){
				String curArg=args[i];
				if (curArg.equalsIgnoreCase("-d")){ //get directory name
					i++;
					dirName=args[i];
				} else if (curArg.equalsIgnoreCase("-n")){
					curArg=args[++i];
					while (! curArg.startsWith("-") && i<args.length){
						curArg=args[i];
						numFeatures.add(curArg);
						i++;
					}
				} else {
					throw new Exception("INVAID OPTION");
				}
			}
		} catch (Exception e){
			    System.out.println(e.getMessage());
				System.out.println("Read the results from a xvalidated mutual information run and" );
				System.out.println("\t return the top most selected features" );
				System.out.println("CountFeatureVotes -d <dirName> -n <numFeatures>" );
				System.out.println("Ex: CountFeatureVotes -d ./3d.K2 -n 25 100 " );
		}
		CountFeatureVotes myCFV = new CountFeatureVotes(dirName,numFeatures);
	}
}
