import java.io.*;
import java.util.*;

/*
 * Created on Sep 2, 2004
 *
 * Createa all single residue mutations given a FASTA file
 */

/**
 * @author Sam Danziger
 *
 */
public class CreateAllSinglePointMuts {
    ArrayList oneLetterCodes;
    String strFasta;
	
	/**
	 * 
	 * @param fileName
	 * @param offSet  The number of the first amino acid (ex: 96 for the p53 core domain)
	 */
    public CreateAllSinglePointMuts (String fileName, int offSet){
		initialize();
		ArrayList aaList = readFasta(fileName);
		strFasta=(String) (aaList.get(0));
		for(int i=1; i<aaList.size(); i++){
			strFasta=strFasta+(String)(aaList.get(i));
		}
		
		/*Write a file for each mutant*/
		for (int i=0; i<aaList.size(); i++){
			String curWT=(String)(aaList.get(i));
			for (int j=0; j<oneLetterCodes.size();j++){
				String curMut=(String)(oneLetterCodes.get(j));
				if (!curWT.equalsIgnoreCase(curMut)){
					printMutant(curWT,i,offSet,curMut);
				}
			}
		}
	}

	/**
	 * Print out a mutant in Jianlin's format:
	 * mutant:   (if not exist, use ?)
	 * sequence: ATPH....
	 * mutation position: 2
	 * original AA: T
	 * substitute AA: W
	 * structure:           (if not exists: use ?, otherwise fileName)
	 * ph value:            (if not exist: use ?)
	 * temperature:         (if not exist: use ?)
	 * solvent_acc:         (if not exist: use ?)
	 * @param fasta
	 * @param position
	 * @param mut
	 */
	private void printMutant(String wt, int position, int offset, String mut){
		try {
			int namePos=position+offset;
			String name = wt+namePos+mut;
			PrintWriter out = new PrintWriter( new FileWriter(name.toLowerCase()+".JC"));
			
			out.println("mutant:\t"+name.toUpperCase());
			out.println("sequence:\t"+strFasta.toUpperCase());
			int mutPosition=position+1; //Star numbering at 1
			out.println("mutant position:\t"+Integer.toString(mutPosition));
			out.println("original AA:\t"+wt.toUpperCase());
			out.println("substitute AA:\t"+mut.toUpperCase());
			out.println("structure:\t"+name.toLowerCase()+"_Qiang.pdb");
			out.println("ph value:\t?");
			out.println("temperature:\t37 C");
			out.println("solvent_acc\t?");
			
			out.close();
		
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	/**
	 * Initialize the data 
	 *
	 */
	private void initialize(){
		oneLetterCodes=new ArrayList();
		String[] aas = {"a","r","n","d","c","e","q","g","h","i","l","k","m","f","p","s","t","w","y","v"};
		for (int i= 0; i<aas.length; i++){
			oneLetterCodes.add(new String(aas[i]));
		}
	}
	
	/**
	 * Read a FASTA file into an array list
	 * @param fileName
	 * @return
	 */
	private ArrayList readFasta(String fileName){
		ArrayList aaList = new ArrayList();
	
		FileReader inFile;
		try {
			inFile = new FileReader(fileName);
			BufferedReader in = new BufferedReader(inFile);
			
			String line = in.readLine();
			while (line.startsWith(">")){
				line = in.readLine();
			}
			char [] fasta = line.toCharArray();
			for (int i=0; i<fasta.length; i++){
   				aaList.add(new String(String.valueOf(fasta[i])));
			}
			
			inFile.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	
		return aaList;
	}

	/**
	 * Create All Single Residue Mutants Given A FASTA WT template
	 * @param args
	 */
	public static void main (String args []) {
		String fileName=null;
		int offSet=0;
		try {
			if (args.length==0){
				throw new Exception("MISSING PARAMETERS");
			}
			for (int i=0;i<args.length;i++){
				String curArg=args[i];
				if (curArg.equalsIgnoreCase("-f")){ //get directory name
					i++;
					fileName=args[i];
				} else if (curArg.equalsIgnoreCase("-o")){
					i++;
					offSet=Integer.parseInt(args[i]);
				} else {
					throw new Exception("INVAID OPTION");
				}
			}
		} catch (Exception e){
			    System.out.println(e.getMessage());
				System.out.println("Create All Single Residue Mutants Given A FASTA WT template" );
				System.out.println("CreateAllSinglePointMuts -f <fileName> -o <fistAAnumber>" );
				System.out.println("Ex: CountFeatureVotes -f p53.fast -o 96" );
				System.exit(0);
		}
		CreateAllSinglePointMuts myCASPM = new CreateAllSinglePointMuts (fileName, offSet);
	}
}
