import java.io.*;
import java.util.*;
/** 
 * @author Sam Danziger
 *
 * Filter all repeats out of .arff file, create a list of it's contents
 * and everythign that's filtered out.
 */
class FilterArff{
	ArrayList mutantList;
	ArrayList exclList;
	
	public FilterArff(String arffName){
		mutantList = new ArrayList();
		exclList = new ArrayList();
		
		try {
			FileReader inFile = new FileReader(arffName);
			BufferedReader in = new BufferedReader(inFile);
			
			String outName = addFiltered(arffName);
			PrintWriter out = new PrintWriter( new FileWriter(outName));
			
			//Header
			String line = in.readLine();
			while (!(line.toLowerCase()).startsWith("@data")){
				out.println(line);
				line = in.readLine();
			}
			out.println(line);
			
			//Data
			line = in.readLine();
			while (line != null){
				if (mutantList.contains(line.toLowerCase())){
					exclList.add(line.toLowerCase());
					line = in.readLine();
				} else {
					mutantList.add(line.toLowerCase());
					out.println(line);
					line = in.readLine();
					out.println(line);
				}
				line = in.readLine();
			}
			out.close();
			in.close();
			writeList(mutantList,arffName+".list");
			writeList(exclList,outName+".repeat.list");			
		} catch (Exception e){
			System.out.println(e.getMessage());
			e.printStackTrace();			
		}
	}
	
	/**
	 * Put the "filtered" into the fileName 
	 * @param fileName
	 * @return
	 */
	private String addFiltered (String fileName){
		String newName = null;
		
		int idx = fileName.lastIndexOf("/");
		if (idx == -1){
			newName = "filtered."+fileName;
		} else {
			newName = fileName.subSequence(0,idx+1)+"filtered."+fileName.subSequence(idx+1,fileName.length());
		}
		return newName;
	}
	
	/**
	 * Write all elements of any collection to a file
	 * @param list
	 * @param fileName
	 */
	private void writeList(Collection list, String fileName){
		try{
			PrintWriter output = new PrintWriter( new FileWriter(fileName));
			Object [] objList = list.toArray();
			for(int i=0;i<objList.length;i++){
				output.println(objList[i]);
			}
			output.close();
			
		}catch (Exception e){
			System.out.println(e.getMessage());
			e.printStackTrace();
		}	
	}
	
	/**
	 * Filter repeats out of a .arff file
	 * FilterArff <arffName>
	 * FilterArff 3d.arff
	 * 
	 * @param args
	 */
	public static void main (String args []) {
		if (args.length != 1){
			System.out.println("Filter repeats out of a .arff file" );
			System.out.println("FilterArff <arffName>" );
			System.out.println("FilterArff 3d.arff" );
		} else {
			String arffName = args[0];
			FilterArff myFilterArff = new FilterArff(arffName);
			System.out.println("Completed Execution: FilterArff "+arffName);
		}
	}
	
}