import java.io.*;
import java.util.*;
/**
 * 
 * @author sad1225
 *
 * Given a list of selected features, filter out the elements not 
 * selected in the .arff file
 */
class FilterFeatures{
	ArrayList featureList;
	ArrayList idxList;
	
	/**
	 * Constructor
	 * @param filterListName
	 * @param arffName
	 */
	public FilterFeatures(String filterListName, String arffName, boolean nameFlag){
		featureList = new ArrayList();
		idxList = new ArrayList();
		readFeatureList(filterListName,nameFlag);
		
		try{
			FileReader inFile = new FileReader(arffName);
			BufferedReader arffReader = new BufferedReader(inFile);

			String outputFileName = "best."+arffName;
			FileWriter outFile = new FileWriter(outputFileName);
			PrintWriter outArff = new PrintWriter(outFile);
			
			String line = arffReader.readLine();
			while (!((line.toLowerCase()).startsWith("@attribute"))){
				outArff.println(line);
				line = arffReader.readLine();
			}

			// Filter out attributes
			int ctr = 1;
			while (!((line.toLowerCase()).startsWith("@data"))){
				if (nameFlag == false) {
					if (featureList.contains(new Integer(ctr))){
						System.out.println(line);
						outArff.println(line);
						idxList.add(new Integer(ctr));
					}
				} else {
					StringTokenizer tok = new StringTokenizer(line);
					String curTok = tok.nextToken();
					curTok = tok.nextToken();
					
					if (featureList.contains(curTok.toLowerCase())){
						System.out.println(line);
						outArff.println(line);
						idxList.add(new Integer(ctr));
					}
				}
				line = arffReader.readLine();
				ctr++;
			}
			
			// Filter out data
			outArff.println(line);
			line = arffReader.readLine();
			while (line !=null){
				if (line.startsWith("%")){
					System.out.println(line);
					outArff.println(line);
					line = arffReader.readLine();
				}

				StringTokenizer tok = new StringTokenizer(line,",");
				ctr = 1;
				while(tok.hasMoreElements()){
					String curTok = tok.nextToken();
					if (idxList.contains(new Integer(ctr))){
						outArff.print(curTok+",");
					}				
					ctr++;
				}
				outArff.println();
				line = arffReader.readLine();
			}
			
			arffReader.close();
			outArff.close();			
		} catch (Exception e){
			e.printStackTrace();
		}		
	}
	
	/**
	 * Read in the list of features to use
	 * @param filterListName
	 */
	private void readFeatureList(String filterListName, boolean nameFlag){
		try{
			FileReader inFile = new FileReader(filterListName);
			BufferedReader listReader = new BufferedReader(inFile);
			
			String line = listReader.readLine();
			line = listReader.readLine();
			
			StringTokenizer tok = new StringTokenizer(line," ");
			String curTok = tok.nextToken();
			tok = new StringTokenizer(curTok,","); 
			while (tok.hasMoreTokens()){
				curTok = tok.nextToken();
				if (nameFlag == false){
					int intCurTok = Integer.parseInt(curTok);
					featureList.add(new Integer(intCurTok));
				} else {
					featureList.add(curTok.toLowerCase());
				}
			}
			listReader.close();			
			
		} catch (Exception e){
			e.printStackTrace();
		}		
	}
	
	/**
	 * Provide command line interface
	 * @param args
	 */	
	public static void main (String args []){
		if (args.length != 2 && args.length != 3){
			System.out.println("Given a list of features, filter a .arff file");
			System.out.println("\tFilterFeatures <filterListName> <arffName> <nameFlag>");
			System.out.println("ex: FilterFeatures aircComplete.full.txt aircComplete.arff");
			System.out.println("Include name flag to filter features by name, not number");
		} else if (args.length == 2){
			FilterFeatures myFilterFeatures = new FilterFeatures(args[0], args[1],false);			
		} else {
			FilterFeatures myFilterFeatures = new FilterFeatures(args[0], args[1],true);			
		}
	}	
}
