import java.io.BufferedReader;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.StringTokenizer;

/*
 * Created on May 26, 2005
 *
 * Filter the MI weights provided.  Filter out residues so that each has
 * only one steric and one electrostatic value per residue
 */

/**
 * @author sdanzige
 *
 */
public class FilterMIselect {
	/**
	 * 
	 * @author sdanzige
	 *
	 * Hold a line selected from MI, break it up for manipulation
	 */
	private class MIfeatures implements Comparable{
		public String featureString;
		public Double miValue; //The MI value
		public String attrLine; //The attribute line
		public String type;  //Distance or PhiElectrostatic
		public Integer residue; //The residue number
		
		private MIfeatures (String _featureString){
			featureString=_featureString;
			StringTokenizer tok = new StringTokenizer(featureString);
			if (tok.countTokens() != 3){
				System.err.println("Improper MI file.  Each line should be in the form");
				System.err.println("<MIvalue> <IDnumber> <AttributeName>");
			} else {
				String strMIvalue=tok.nextToken();
				miValue=Double.valueOf(strMIvalue);
				String IDnum=tok.nextToken();
				attrLine=tok.nextToken();
				StringTokenizer attTok=new StringTokenizer(attrLine,"_");
				
				String strType=attTok.nextToken();
				//Remove the trailing digits from the label.
				strType=strType.replaceAll("[0-9]*$","");
				type=strType;
				
				String resType=attTok.nextToken();
				String resNum=attTok.nextToken();
				residue=new Integer(Integer.parseInt(resNum));
			}
		}
		
		/**
		 * Compare based on miValue  
		 */
		public int compareTo(Object o){
			MIfeatures that = (MIfeatures)o;
			int retValue=this.miValue.compareTo(that.miValue);
			
			return retValue;
		}
		
		/**
		 * ID = "<residue> <type>"
		 * @return
		 */
		public String ID(){
			String idVal=residue+" "+type;
			return idVal;
		}
		
		public String toString(){
			String retValue=attrLine+"\t"+residue+"\t"+type+"\t"+miValue;
			return retValue;			
		}
	}

    private ArrayList miFeatureList; //Store the array of attributes here
    
    /**
     * Select the best MI features
     * @param fileName
     */
	public FilterMIselect(String fileName){
		//Load the file
		loadFile(fileName); 
		
		//Select the features from each residue with the highest MI value
		ArrayList sortedFeatures=choseFeatures(miFeatureList);
		
		//Print out the list of features
		print(sortedFeatures);
	}

	/**
	 * Print out the list of features in a convienient form
	 * @param sortedFeatures
	 */
	private void print(ArrayList sortedFeatures){
		for (int i=0; i<sortedFeatures.size(); i++){
			MIfeatures curFeature=(MIfeatures)(sortedFeatures.get(i));
			System.out.println(curFeature);
		}
	}

	/**
	 * Chose the best features
	 * @param _miFeatureList
	 * @return
	 */
	private ArrayList choseFeatures (ArrayList _miFeatureList){
		ArrayList newFeatures = new ArrayList();
		
		//Find the residue numbers/type pairs listed
		ArrayList resNumbers=new ArrayList();
		for(int i=0; i<_miFeatureList.size(); i++){
			MIfeatures curFeature=(MIfeatures)_miFeatureList.get(i);
			if(!resNumbers.contains(curFeature.ID())) {
				resNumbers.add(curFeature.ID());
			}
		}
		Collections.sort(resNumbers);
		
		//Pick the highest MI value for each residue number/type pair
		for (int i=0; i<resNumbers.size(); i++){
			String curResNumber=(String)(resNumbers.get(i));
			
			//Contruct the list for this residue number/type 
			ArrayList resNumSpecificFeature=new ArrayList();
			for (int j=0; j<_miFeatureList.size(); j++){
				MIfeatures curFeature=(MIfeatures)_miFeatureList.get(j);
				String curID=curFeature.ID();
				if (curResNumber.equals(curID)){
					resNumSpecificFeature.add(curFeature);
				}
			}
			
			//Sort it and pick the best
			Collections.sort(resNumSpecificFeature); //sort ascending
			MIfeatures bestFeature=(MIfeatures)(resNumSpecificFeature.get(resNumSpecificFeature.size()-1));
			newFeatures.add(bestFeature);
		}
		
		return newFeatures;		
	}

	/**
	 * Load the file
	 * @param fileName
	 */
	private void loadFile(String fileName) {
		miFeatureList = new ArrayList();
		try {
			FileReader inFile = new FileReader(fileName);
			BufferedReader in = new BufferedReader(inFile);
			
			String line=in.readLine();
			while(line!=null){
				MIfeatures newFeatures=new MIfeatures(line);
				miFeatureList.add(newFeatures);
				line=in.readLine();
			}
			
		} catch (Exception e){
			System.out.println(e.getMessage());
			e.printStackTrace();
		}		
	}
	
	/**
	 * The main
	 * @param args
	 */
	public static void main (String args []) {
		if ((args.length != 1)){
			System.out.println("Filter the MI weights provided." );
			System.out.println("FilterMIselect <fileName>" );
			System.out.println("FilterMIselect 2dMIvalues" );
		} else {
			String arffName = args[0];
			FilterMIselect myFilterMIselect = new FilterMIselect(arffName);
		}
	}
}
