import java.io.*;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.StringTokenizer;
/**
 * 
 * @author Sam Danziger
 *
 * Create the 1d.arff file for a given mutant
 */
class MakeArff1d_v2 {

	class Region {
		String name;
		
		//Size
		int small2small;
		int small2med;
		int small2large;
		int med2small;
		int med2med;
		int med2large;
		int large2small;
		int large2med;
		int large2large;
		
		//Aromatic
		int aro2aro;
		int aro2naro;
		int naro2aro;
		int naro2naro;
		
		//Charge
		int pos2pos;
		int pos2neg;
		int pos2unc;
		int neg2pos;
		int neg2neg;
		int neg2unc;
		int unc2pos;
		int unc2neg;
		int unc2unc;
		
		//Hydophobicity
		int phobic2philic;
		int phobic2phobic;
		int philic2philic;
		int philic2phobic;
		
		//Polarity
		int pol2npol;
		int pol2pol;
		int pol2ind;
		int npol2npol;
		int npol2pol;
		int npol2ind;
		int ind2npol;
		int ind2pol;
		int ind2ind;
		
		public Region(String regionName){
			name = regionName;
			
			//Size
			small2small=0;
			small2med=0;
			small2large=0;
			med2small=0;
			med2med=0;
			med2large=0;
			large2small=0;
			large2med=0;
			large2large=0;
		
			//Aromatic
			aro2aro=0;
			aro2naro=0;
			naro2aro=0;
			naro2naro=0;
		
			//Charge
			pos2pos=0;
			pos2neg=0;
			pos2unc=0;
			neg2pos=0;
			neg2neg=0;
			neg2unc=0;
			unc2pos=0;
			unc2neg=0;
			unc2unc=0;
		
			//Hydophobicity
			phobic2philic=0;
			phobic2phobic=0;
			philic2philic=0;
			philic2phobic=0;
		
			//Polarity
			pol2npol=0;
			pol2pol=0;
			pol2ind=0;
			npol2npol=0;
			npol2pol=0;
			npol2ind=0;
			ind2npol=0;
			ind2pol=0;
			ind2ind=0;
		}
		
		public String print(){
			String out = small2small+","+small2med+","+small2large+","+med2small+","+med2med+",";
			out = out+med2large+","+large2small+","+large2med+","+large2large+",";
			out = out+aro2aro+","+aro2naro+","+naro2aro+","+naro2naro+","+pos2pos+",";
			out = out+pos2neg+","+pos2unc+","+neg2pos+","+neg2neg+","+neg2unc+","+unc2pos+",";
			out = out+unc2neg+","+unc2unc+","+phobic2philic+","+phobic2phobic+","+philic2philic+",";
			out = out+philic2phobic+","+pol2npol+","+pol2pol+","+pol2ind+","+npol2npol+",";
			out = out+npol2pol+","+npol2ind+","+ind2npol+","+ind2pol+","+ind2ind+",";
			return out;
		}
	}
	
	PrintWriter output;
	
	/*
	Region other;
	Region beta_sandwich;
	Region DNA_Bind1;
	Region DNA_Bind2;
	Region DNA_Bind3;
	Region Zn_Bind1;
	Region Zn_Bind2;
	*/
	ArrayList regions; //Store the count for each region here
	boolean isExcluded; //Set to true if DNA or zinc binding region
	
	public MakeArff1d_v2(String mutName, String outFilename){
		regions = new ArrayList();
		isExcluded = false;
		String [] regionNames = {"other","S1","S2","S2p","S3","S4","S5","S6","S7","S8","S9","S10"};
		
		for (int i=0; i<regionNames.length; i++){
			regions.add(new Region(regionNames[i]));
		}			

						
		try {
			output = new PrintWriter( new FileWriter(outFilename));
			
			//Loop through each mutant separated by _
			StringTokenizer tok = new StringTokenizer(mutName,"_");
			while(tok.hasMoreTokens()){
				/*Need to change here*/
				processMut(tok.nextToken());
			}
			
			//Print out the .arff file
			Calendar myDate = new GregorianCalendar();
			output.println("@RELATION\t2D_"+ (myDate.get(Calendar.MONTH)+1)+"/"+myDate.get(Calendar.DATE)+"/"+myDate.get(Calendar.YEAR)+"_"+myDate.get(Calendar.HOUR_OF_DAY)+":"+myDate.get(Calendar.MINUTE));
			
			for (int i=0; i<regionNames.length; i++){
				printHeader(regionNames[i]);
			}			
			
			output.println("@DATA");
			output.println("%"+mutName);
			for (int i=0; i<regions.size(); i++){
				output.print(((Region)(regions.get(i))).print());
			}
			
			output.close();
		} catch (IOException e) {			
			e.printStackTrace();
		}
	}
	
	
	private void printHeader(String prefix){
		prefix = prefix+"_";
	  	output.println("@ATTRIBUTE\t"+prefix+"small2small\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"small2med\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"small2large\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"med2small\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"med2med\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"med2large\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"large2small\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"large2med\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"large2large\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"aro2aro\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"aro2naro\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"naro2aro\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"naro2naro\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"pos2pos\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"pos2neg\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"pos2unc\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"neg2pos\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"neg2neg\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"neg2unc\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"unc2pos\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"unc2neg\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"unc2unc\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"phobic2philic\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"phobic2phobic\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"philic2philic\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"philic2phobic\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"pol2npol\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"pol2pol\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"pol2ind\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"npol2npol\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"npol2pol\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"npol2ind\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"ind2npol\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"ind2pol\tNUMERIC");
		output.println("@ATTRIBUTE\t"+prefix+"ind2ind\tNUMERIC");
	}
	
	
	private String findRegion(int aaNum){
		String curRegion;		  
//		Set curRegion to apropriate region
		
///

		//if ( aaNum >= 171  && aaNum <= 184){
		if ( aaNum >= 174  && aaNum <= 181){
		  	curRegion = "Zn_Bind1";
		//} else if ( aaNum >= 233 && aaNum <= 244) {
		} else if ( aaNum >= 236 && aaNum <= 241) {
		  	curRegion = "Zn_Bind2";
		//} else if ( aaNum >= 278 && aaNum <= 286 ) {
		} else if ( aaNum >= 281 && aaNum <= 283 ) {
		  	curRegion = "DNA_Bind1";
		//} else if ( aaNum >= 245 && aaNum <= 251 ) {
		} else if ( aaNum >= 247 && aaNum <= 249 ) {
			curRegion = "DNA_Bind2";
		//} else if ( aaNum <= 123 && aaNum >= 117 ) {
		} else if ( aaNum >= 119 && aaNum <= 121 ) {
			curRegion = "DNA_Bind3";
		} else if ( aaNum >= 110 && aaNum <= 112 ) {
		  //s1
		  	curRegion = "S1";
		} else if ( aaNum >= 122 && aaNum <= 127 ) {
		  //s2
		  	curRegion = "S2";
		 } else if ( aaNum >= 132 && aaNum <= 135 ) {
		  //s2'
		  	curRegion = "S2p";
		 } else if ( aaNum >= 141 && aaNum <= 146 ) {
		  //s3
		  	curRegion = "S3";
		 } else if ( aaNum >= 156 && aaNum <= 163 ) {
		  //s4
		  	curRegion = "S4";
		 } else if ( aaNum >= 195 && aaNum <= 198 ) {
		  //s5
		  	curRegion = "S5";
		 } else if ( aaNum >= 204 && aaNum <= 207 ) {
		  //s6
			  curRegion = "S6";
		 } else if ( aaNum >= 214 && aaNum <= 219 ) {
		  //s7
			  curRegion = "S7";
		 } else if ( aaNum >= 230 && aaNum <= 235 ) {
			  //s8
			  curRegion = "S8";
		 } else if ( aaNum >= 250 && aaNum <= 258 ) {
			  //s9
			  curRegion = "S9";
		 } else if ( aaNum >= 264 && aaNum <= 274 ) {
			  //s10
			  curRegion = "S10";
		  } else {
			  //All other regions
			  curRegion = "other";
		  }
		  
		  return curRegion;
	}
	
	/**
	 * Set the flags based on each mutant
	 * @param mutant
	 */
	private void processMut(String mutant){
		Region curRegion = null;
		
		//Parse name
		char wtAA = mutant.charAt(0);
		char mutAA = mutant.charAt(mutant.length()-1);
		String sAAnum = mutant.substring(1,mutant.length()-1);
		int aaNum = Integer.parseInt(sAAnum);
		
		//Set curRegion to apropriate region
		String mutRegion = findRegion (aaNum);
		if (mutRegion.startsWith("Zn_Bind") || mutRegion.startsWith("DNA_Bind")) {
			output.println("EXCLUDED");
			output.println(mutant);
			output.println(mutRegion);	
			isExcluded = true;						
		} else {
			for (int i=0; i<regions.size(); i++){
				Region tstRegion = (Region)(regions.get(i));
				if (mutRegion.equals(tstRegion.name)){
					curRegion = tstRegion;
					break;
				}
			}
		
			//Set Correct properties
		
			//SIZE CHANGE
			int wtSize = aaSize(wtAA);
			int mutSize = aaSize(mutAA);
		
			if (wtSize == 1 && mutSize ==1){
				curRegion.large2large++;
			} else if (wtSize == 1 && mutSize ==0){
				curRegion.large2med++;
			} else if (wtSize == 1 && mutSize ==-1){
				curRegion.large2small++;
			} else if (wtSize == 0 && mutSize ==1){
				curRegion.med2large++;
			} else if (wtSize == 0 && mutSize ==0){
				curRegion.med2med++;
			} else if (wtSize == 0 && mutSize ==-1){
				curRegion.med2small++;
			} else if (wtSize == -1 && mutSize ==1){
				curRegion.small2large++;
			} else if (wtSize == -1 && mutSize ==0){
				curRegion.small2med++;
			} else if (wtSize == -1 && mutSize ==-1){
				curRegion.small2small++;
			}
		
			//AROMATIC
			int wtAro = aromatic(wtAA);
			int mutAro = aromatic(mutAA);
		
			if (wtAro==1 && mutAro==1) {
				curRegion.aro2aro++;
			} else if (wtAro==1 && mutAro==-1) {
				curRegion.aro2naro++;
			} else if (wtAro==-1 && mutAro==1) {
				curRegion.naro2aro++;
			} else if (wtAro==-1 && mutAro==-1) {
				curRegion.naro2naro++;
			} 
		
		
			//CHARGE
			int wtCharge = charge(wtAA);
			int mutCharge = charge(mutAA);
		
			if (wtCharge==1 && mutCharge==1){
				curRegion.pos2pos++;
			} else if (wtCharge==1 && mutCharge==0){
				curRegion.pos2unc++;
			} else if (wtCharge==1 && mutCharge==-1){
				curRegion.pos2neg++;
			} else if (wtCharge==0 && mutCharge==1){
				curRegion.unc2pos++;
			} else if (wtCharge==0 && mutCharge==0){
				curRegion.unc2unc++;
			} else if (wtCharge==0 && mutCharge==-1){
				curRegion.unc2neg++;
			} else if (wtCharge==-1 && mutCharge==1){
				curRegion.neg2pos++;
			} else if (wtCharge==-1 && mutCharge==0){
				curRegion.neg2unc++;
			} else if (wtCharge==-1 && mutCharge==-1){
				curRegion.neg2neg++;
			}
		
			//HYDROPHOBICITY:  Uses Engelman, Steitz, and GoldMan
			int wtHydro = hydrophobicity(wtAA);
			int mutHydro = hydrophobicity(mutAA);
		
			if (wtHydro==1 && mutHydro==1){
				curRegion.phobic2phobic++;
			} else if (wtHydro==1 && mutHydro==-1){
				curRegion.phobic2philic++;
			} else if (wtHydro==-1 && mutHydro==1){
				curRegion.philic2phobic++;
			} else if (wtHydro==-1 && mutHydro==-1){
				curRegion.philic2philic++;
			}
		
			//POLARITY
			int wtPol = polarity(wtAA);
			int mutPol = polarity(mutAA);
			if (wtPol==1 && mutPol ==1) {
				curRegion.pol2pol++;
			} else if (wtPol==1 && mutPol ==0) {
				curRegion.pol2ind++;
			} else if (wtPol==1 && mutPol ==-1) {
				curRegion.pol2npol++;
			} else if (wtPol==0 && mutPol ==1) {
				curRegion.ind2pol++;
			} else if (wtPol==0 && mutPol ==0) {
				curRegion.ind2ind++;
			} else if (wtPol==0 && mutPol ==-1) {
				curRegion.ind2npol++;
			} else if (wtPol==-1 && mutPol ==1) {
				curRegion.npol2pol++;
			} else if (wtPol==-1 && mutPol ==0) {
				curRegion.npol2ind++;
			} else if (wtPol==-1 && mutPol ==-1) {
				curRegion.npol2npol++;
			}
		}
		
	}
	
	/**
	 * 
	 * @param aa
	 * @return 1 = large, -1 = small, 0 = medium 
	 */
	private int aaSize(char aa){
		char [] smallAA = {'a','g','s'};
		char [] largeAA = {'f','h','k','r','w','y'};
		for (int i =0; i<largeAA.length; i++){
			if ( aa == largeAA[i] ){
				return 1;
			}
		}

		for (int i =0; i<smallAA.length; i++){
			if ( aa == smallAA[i] ){
				return 1;
			}
		}

				
		return 0;
	}
	
	/**
	 * 
	 * @param aa
	 * @return 1 == aromatic, -1 not aromatic
	 */
	private int aromatic (char aa){
		char [] aro = {'f','w','y'};
		for (int i =0; i<aro.length; i++){
			if ( aa == aro[i] ){
				return 1;
			}
		}
		
		return -1;
	}
	
	/**
	 * 
	 * @param aa
	 * @return 1 = positive, -1 = negative, 0 = neutral
	 */
	private int charge (char aa){
		char [] pos = {'h','k','r'};
		char [] neg = {'d','e'};
		for (int i =0; i<pos.length; i++){
			if ( aa == pos[i] ){
				return 1;
			}
		}

		for (int i =0; i<neg.length; i++){
			if ( aa == neg[i] ){
				return -1;
			}
		}
		
		
		return 0;
	}
	
	/**
	 * Determine the hydrophobicity of an amino acid
	 * @param aa
	 * @return 1 if hydrophobic, -1 if not
	 */
	private int hydrophobicity (char aa){
		char [] hydrophobic = {'f','m','i','l','v','c','w','a','t','g','s'};
		for (int i =0; i<hydrophobic.length; i++){
			if ( aa == hydrophobic[i] ){
				return 1;
			}
		}
		
		return -1;		
	}
	
	/**
	 * Determine the polarity of an amino acid
	 * @param aa
	 * @return 1 = polar, 0 = indifferent, -1 = nonpolar
	 */
	private int polarity (char aa){
		char [] polar = {'r','k','d','e','n','q'};
		char [] ind = {'h','a','y','t','s','p','g'};
		
		for (int i =0; i<polar.length; i++){
			if ( aa == polar[i] ){
				return 1;
			}
		}
		
		for (int i =0; i<ind.length; i++){
			if ( aa == ind[i] ){
				return 0;
			}
		}
		
		return -1;
	}

	
	/**
	 * Read in the alpha coordinate file and the energy file to generate a 
	 * 1d.arff file 
	 *	Ex: MakeArff1d  <mutant> <output>
	 *
	 * @param args
	 */
	public static void main (String args []) {

		if (args.length != 2){
			System.out.println("Read in the alpha coordinate file and the energy file to generate a 3d.arff file" );
			System.out.println("MakeArff1d <Mutant> <Outfile>" );
			System.out.println("Ex: MakeArff1d  r273l_n235k r273l_n235k.arff");
		} else {
			MakeArff1d_v2 myMA = new MakeArff1d_v2(args[0],args[1]);
		}
	}
}
