import java.io.*;
import java.text.NumberFormat;
import java.util.*;
/**
 * 
 * @author Sam Danziger
 *
 * Read in the alpha coordinate file and the energy file to generate a 
 * 3d.arff file
 */
class MakeArff3d {
	PrintWriter output;
    
	
	public MakeArff3d(String alphaFilename, String energyFilename, String outFilename){
		try {
			output = new PrintWriter( new FileWriter(outFilename));
			printHeader();
			output.print(findCoords(alphaFilename)+findEnegies(energyFilename));
			
			output.close();
		} catch (IOException e) {			
			e.printStackTrace();
		}
	}
	
	private void printHeader(){
		Calendar myDate = new GregorianCalendar();
		output.println("@RELATION\t3D_"+ (myDate.get(Calendar.MONTH)+1)+"/"+myDate.get(Calendar.DATE)+"/"+myDate.get(Calendar.YEAR)+"_"+myDate.get(Calendar.HOUR_OF_DAY)+":"+myDate.get(Calendar.MINUTE));
		for (int i = 96; i < 290; i++){
			output.println("@ATTRIBUTE aa"+i+"X NUMERIC");
			output.println("@ATTRIBUTE aa"+i+"Y NUMERIC");
			output.println("@ATTRIBUTE aa"+i+"Z NUMERIC");			
		}
		
		output.println("@ATTRIBUTE angE NUMERIC");
		output.println("@ATTRIBUTE dihedE NUMERIC");
		output.println("@ATTRIBUTE vdwE NUMERIC");
		output.println("@ATTRIBUTE eelE NUMERIC");
		output.println("@ATTRIBUTE hbondE NUMERIC");
		output.println("@ATTRIBUTE vdw14E NUMERIC");
		output.println("@ATTRIBUTE eel14E NUMERIC");
		output.println("@ATTRIBUTE restE NUMERIC");
		output.println("@DATA");
	}
	
	private String findCoords (String alphaFilename){
		String data="";
		try {
			FileReader inFile = new FileReader(alphaFilename);
			BufferedReader in = new BufferedReader(inFile);
			
			String line = in.readLine();
			line = in.readLine();
			
			line = in.readLine();
			
			NumberFormat myFormat = NumberFormat.getInstance();
			myFormat.setMaximumFractionDigits(6);
			
			while (line != null){
				StringTokenizer tok = new StringTokenizer(line);
				
				double x = Double.parseDouble(tok.nextToken());
				double y = Double.parseDouble(tok.nextToken());
				double z = Double.parseDouble(tok.nextToken());
				double mag = Double.parseDouble(tok.nextToken());
				
				x = x*mag;
				y = y*mag;
				z = z*mag;
				
				data = data + myFormat.format(x) +","+myFormat.format(y)+","+myFormat.format(z)+",";
				
				line = in.readLine();
			}
			
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return data;	
	}
	
	private String findEnegies (String energyFilename){
		String data = "";
		
		try {
			FileReader inFile = new FileReader(energyFilename);
			BufferedReader in = new BufferedReader(inFile);
			
			String line = in.readLine();
			while (line.indexOf("FINAL RESULTS")==-1){
				line = in.readLine();		
			}
			
			while (! line.startsWith(" BOND")){
				line = in.readLine();		
			}
			
			StringTokenizer tok = new StringTokenizer(line);
			
			tok.nextToken();
			tok.nextToken();
			String BOND = tok.nextToken();
			tok.nextToken();
			tok.nextToken();
			String ANGLE = tok.nextToken();
			tok.nextToken();
			tok.nextToken();
			String DIHED = tok.nextToken(); 
			data = data + parseEnergy(BOND) +","+ parseEnergy(ANGLE) +","+ parseEnergy(DIHED)+",";
			
			line = in.readLine();
			tok = new StringTokenizer(line);
			tok.nextToken();
			tok.nextToken();
			String VDWAALS = tok.nextToken();
			tok.nextToken();
			tok.nextToken();
			String EEL = tok.nextToken();
			tok.nextToken();
			tok.nextToken();
			String EGB = tok.nextToken(); 
			data = data + parseEnergy(VDWAALS) +"," + parseEnergy(EEL) +"," + parseEnergy(EGB)+",";
			
			line = in.readLine();
			tok = new StringTokenizer(line);
			tok.nextToken();
			tok.nextToken();
			tok.nextToken();
			String VDW14 = tok.nextToken();
			tok.nextToken();
			tok.nextToken();
			tok.nextToken();
			String EEL14 = tok.nextToken();
			data = data + parseEnergy(VDW14)+"," + parseEnergy(EEL14)+",";
			

		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			System.err.println("Error in: "+energyFilename);
			e.printStackTrace();
		}

		return data;
	}
	
	/**
	 * Parse the energy string and replace errors (*****) with ? 
	 * @param energy
	 * @return
	 */
	private String parseEnergy(String energy){
		String newEnergy = energy;
		try {
			Double test = Double.valueOf(energy);
		} catch (NumberFormatException e){
			newEnergy="?";
		}
		return newEnergy;
	}
	
	/**
	 * Read in the alpha coordinate file and the energy file to generate a 
 	 * 3d.arff file 
	 *	Ex: MakeArff3d  ./intermediate/r273l/r273l_Qiang_Small_Vect.txt
	 *					../pdbs/r273l/r273l.0.out
	 *					./intermediate/r273l/r273l.arff
	 *
	 * @param args
	 */
	public static void main (String args []) {

		if (args.length != 3){
			System.out.println("Read in the alpha coordinate file and the energy file to generate a 3d.arff file" );
			System.out.println("MakeArff3d <Alpha Filename> <Energy Filename> <Output Filename>" );
			System.out.println("Ex: MakeArff3d  ./intermediate/r273l/r273l_Qiang_Small_Vect.txt ../pdbs/r273l/r273l.0.out ./intermediate/r273l/r273l.arff" );
		} else {
			MakeArff3d myMA = new MakeArff3d(args[0],args[1],args[2]);
		}
	}
}
