import java.io.*;
import java.util.*;

/**
 * 
 * @author Sam
 *
 * Convert a list of p53 mutants into FASTA format
 */
class MakeFasta{
	//String p53Seq;  //The p53 FASTA sequence
	//String mutSeq;   //The mutated sequence
	//ArrayList mutNames; //The list of mutant names
	
	/**
	 * Constructor
	 */
	public MakeFasta(String fileName, String defFile, String lblNum){
		String p53Seq = readFASTA(fileName);
		
		FileReader inputFile;
		try {
			inputFile = new FileReader(defFile);
			BufferedReader defIn = new BufferedReader(inputFile);

			int idx=1;
			String mutation = defIn.readLine();
			while (mutation != null){
				String mutSeq = p53Seq;
			
				mutation=mutation.replaceAll("[ \t].+$",""); // strip off the activity
				StringTokenizer tok = new StringTokenizer(mutation,"_");
				while (tok.hasMoreTokens()) {
					String mut = tok.nextToken();
					mutSeq = mutate(mutSeq,mut);
				}	
				
				//>0_d1hrm__ 1.1.1.1.8 Myoglobin [horse (Equus caballus)]
				System.out.println(">0_d1tsr_b 1.1.1."+lblNum+"."+idx+" "+mutation+" [human (H.S.)]");
				//printFastaLine(">0_d1tsr_b 1.1.1."+lblNum+"."+idx+" "+mutation+" [human (Homo sapiens)]");
				printFastaLine(mutSeq);

				mutation = defIn.readLine();
				idx++;
			}
			
			inputFile.close();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		
		//makeMutants("aircComplete.list","aircComplete.list.truth");
		//printFasta("aircMuts.fasta");
	}
	
	/**
	 * Write out lines as if for the fasta format, 80 characters per line
	 * @param line
	 */
	private void printFastaLine (String line){
		int fastaSize=80;
		
		boolean keepGoing=true;
		do {
			if (line.length() <= fastaSize) {
				System.out.println(line);
				keepGoing=false;
			} else {
				String smallLine = line.substring(0,fastaSize-1);
				System.out.println(smallLine);
				line = line.substring(fastaSize);
			}
		} while (keepGoing);
		
	}
	
	
	/**
	 * Given a list of mutants and their activities, create the mutant FASTA lists
	 * @param mutList
	 * @param mutActive
	 */
	/*
	private void makeMutants(String mutList, String mutActive){
		try{
			FileReader inputFile = new FileReader(mutList);
			BufferedReader mList = new BufferedReader(inputFile);
			
			FileReader inputFile2 = new FileReader(mutActive);
			BufferedReader aList = new BufferedReader(inputFile2);

			String mutName = mList.readLine();
			String mutTruth = aList.readLine();
			int mutCount=0;
			while (mutName != null){
				mutNames.add(mutName + " " + mutTruth);
				
				StringTokenizer tok = new StringTokenizer(mutName);
				String curSeq = p53Seq;
				while (tok.hasMoreTokens()){
					String curMut = tok.nextToken();
					curSeq = mutate(curSeq,curMut);					
				}
				mutSeq.add(curSeq);
				
				mutName = mList.readLine();
				mutTruth = aList.readLine();
			}
			
			mList.close();
			aList.close();
		} catch (Exception e){
			System.out.println(e.getMessage());
			System.out.println(e.getStackTrace());
		}

	}
	*/
	/**
	 * Print out the FASTA file
	 * @param filename
	 */
	
	/*
	public void printFasta(String filename){

		try{ 		
			FileWriter outFile = new FileWriter(filename);
			BufferedWriter out = new BufferedWriter(outFile);
			PrintWriter pw = new PrintWriter (out);
			
			pw.println(">WT 1");
			pw.println(p53Seq);

			for (int i = 0; i < mutSeq.size(); i++){
				pw.println(">"+mutNames.get(i));
				pw.println(mutSeq.get(i));
			}
			pw.close();
		} catch (Exception e){
			System.out.println(e.getMessage());
			System.out.println(e.getStackTrace());
		}
	}
	*/
	
	/**
	 * Read in a file in fasta format.  DOES NOT IGNOR NEWLINE
	 * @param fileName
	 * @return
	 */
	private String readFASTA(String fileName){
		String fasta = null;
		
		try{
			FileReader inputFile = new FileReader(fileName);
			BufferedReader in = new BufferedReader(inputFile);
			
			String line = in.readLine();
			while (line.startsWith(">")){
				line = in.readLine();
			}
			fasta = line;

			in.close();
		} catch (Exception e){
			System.out.println(e.getMessage());
			System.out.println(e.getStackTrace());
		}
		
		return fasta;
	}
	
	/**
	 * Mutate sequence with mutation
	 * @param sequence FASTA
	 * @param mutation Form r273l
	 * @return
	 */
	private String mutate(String sequence, String mutation){
		char wtRes;
		char mutRes;
		int number;
		
		sequence = sequence.toUpperCase();
		mutation = mutation.toUpperCase();
		
		wtRes = mutation.charAt(0);
		mutRes = mutation.charAt(mutation.length()-1);
		String num = mutation.substring(1,mutation.length()-1);
		number = Integer.parseInt(num);
		
		char [] charSeq = sequence.toCharArray();
		charSeq[number-96] = mutRes;
		
		String newSeq = new String(charSeq);
		return newSeq; 
	}
	
	public static void main(String args []){
		if (args.length != 3){
			System.out.println("Create a FASTA sequence for the mutant");
			System.out.println("\tBayes <fastaFile> <defFile> <lblNum - used to help create unique labels>");
			System.out.println("ex: p53.fasta mutants.def 3");
		} else {
			String fileName = args[0];
			String defFile = args[1];
                        String lblNum = args[2];
		
			MakeFasta myP53 = new MakeFasta(fileName,defFile,lblNum);
			
			//String seq = myP53.readFASTA(fileName);
			//String newSeq = myP53.mutate(seq,mutation);
			//System.out.println(seq);
			//System.out.println(newSeq);
		
			//System.out.println((myP53.mutNames).size());
			//System.out.println(myP53.mutNames);
			//System.out.println(myP53.mutSeq);
		}
	}

}
