import java.io.*;
import java.util.*;
/**
 * 
 * @author sad1225
 *
 * Given a list of residues, filter the 2d data set to decrease the resolution
 * of 2d features taht are not in the selected regions.
 */
class Modify2dResolution{
	ArrayList featureList; //The list of features to keep at high resolution
	ArrayList attributeList; //Fill this with the attribute values
	Hashtable distHash; //the Distance hash table using AA acid as key (ILE_112)
	Hashtable elecHash; //the PhiElectrostatic hash table using AA acid as key (ILE_112)
	
	/**
	 * Constructor
	 * @param filterListName
	 * @param arffName
	 */
	public Modify2dResolution(String residueListName, String arffName){
		
		//Read in the residues to preserve
		featureList=readResidueList(residueListName);
		
		//Read in the attribute lists
		parseArffFile(arffName);
		
		//Process the attributes to correctly compress the 2d image
		processAttributes();
		
		//Print out the new data, processing each line on the fly
		echoData(arffName);
	}
	
	/**
	 * Echo the data lines, processing each line on the fly
	 *
	 */
	private void echoData(String arffFileName){
		try{
			FileReader inFile = new FileReader(arffFileName);
			BufferedReader arffReader = new BufferedReader(inFile);

			String line = arffReader.readLine();
			// Read in all attributes
			while (!((line.toLowerCase()).startsWith("@data"))){
				line = arffReader.readLine();
			}
			
			System.out.println("@data");
			line = arffReader.readLine();
			while (line !=null){
				if (line.startsWith("%")){
					System.out.println(line);
					line = arffReader.readLine();
				} else {
					int dataCtr=0;
					String strDataLine[]=line.split(",");
					//Error check to make sure there are the correct # of data?

					//Process this data line
					ArrayList hashes=new ArrayList();
					hashes.add(distHash);
					hashes.add(elecHash);
					
					for (int hashIdx=0; hashIdx < hashes.size(); hashIdx++){
						Hashtable curHash=(Hashtable)(hashes.get(hashIdx));
						
						Enumeration keys=curHash.keys();
						while(keys.hasMoreElements()){
							Object curKey=keys.nextElement();
							String curAA[]=((String)(curKey)).split("_");
							Integer curAANum=new Integer(Integer.parseInt(curAA[1]));

							ArrayList curIdxs=(ArrayList)(curHash.get(curKey));
							if (featureList.contains(curAANum)){
								//Don't compress this data
								for (int i=0;i<curIdxs.size();i++){
									int curIdx=((Integer)(curIdxs.get(i))).intValue();
									System.out.print(strDataLine[curIdx]+",");
								}
								dataCtr = dataCtr+curIdxs.size();
							} else {
								//Compress this data
								String testDataVal=strDataLine[0];
								if (testDataVal.equalsIgnoreCase("?")){
									System.out.print("?"+",");
								} else {
									double cumVal=0;
									for (int i=0;i<curIdxs.size();i++){
										int curIdx=((Integer)(curIdxs.get(i))).intValue();
										String curDataVal=strDataLine[curIdx];
										double curVal=Double.parseDouble(curDataVal);
										cumVal=cumVal+curVal;
									}
									double aveVal=cumVal/curIdxs.size();
									System.out.print(aveVal+",");
								}
								dataCtr = dataCtr+1;
							}
						}						
					}
					System.out.println("");
					//System.err.println("dataCtr= "+dataCtr);
					line = arffReader.readLine();
				}
			}
			arffReader.close();
		} catch (Exception e){
			e.printStackTrace();
		}				
	}
	
	/**
	 * Record the attributes in a AA hashes, one for Distance, the other for
	 * PhiElectrostatics
	 * @return
	 */
	private void processAttributes(){
		//Create the hash tables
		distHash = new Hashtable(); //store the distances here
		elecHash = new Hashtable(); //store the Phi Electrostatics here
		int offset=96; //Core domain starts at residue 96
		
		for (int i=0; i<attributeList.size(); i++){
			String curAttr[]=((String)(attributeList.get(i))).split("_");
			//Filter out Chlorine ions (as they're only there for charge balance)
			if ((curAttr.length == 3) && (curAttr[1].toLowerCase()).indexOf("cl-")==-1){ //Make sure that each line has a valid AA
				String resID=curAttr[1]+"_"+(Integer.parseInt(curAttr[2])+offset);
				if ((curAttr[0].toLowerCase()).startsWith("distance")){
					if (!distHash.containsKey(resID)){
						distHash.put(resID,new ArrayList());
					}
					((ArrayList)(distHash.get(resID))).add(new Integer(i));
				} else {
					if (!elecHash.containsKey(resID)){
						elecHash.put(resID,new ArrayList());
					}
					((ArrayList)(elecHash.get(resID))).add(new Integer(i));
				}
			}
		}
		
		//Create the new attribute list, using the feature list
		ArrayList newAttr = parseAttType(distHash, "Distance");
		newAttr.addAll(parseAttType(elecHash, "PhiElectrostatic"));

		//Print out the new Attribute List
		for (int i=0; i<newAttr.size();i++){
			String curAttr=(String)(newAttr.get(i));
			
			System.out.println("@attribute "+curAttr+" real");
		}
		System.err.println("Number of @attributes: "+newAttr.size());
	}

	/**
	 * Create an attribute list for Distance or PhiElectrostatic
	 * @param hash
	 * @param hashName
	 */
	private ArrayList parseAttType(Hashtable hash, String hashName){
		ArrayList newAttr=new ArrayList();
		
		Enumeration keys=hash.keys(); 
		while(keys.hasMoreElements()){
			Object curKey=keys.nextElement();
			String curAA[]=((String)(curKey)).split("_");
	
			ArrayList curIdxs=(ArrayList)(hash.get(curKey));//Need to store an arrayList with each hashKey
			Integer curAANum=new Integer(Integer.parseInt(curAA[1]));
			if (featureList.contains(curAANum)){
				//If this feature should remain at high resolution
				for (int i=0; i<curIdxs.size(); i++){
						Integer curIdx=(Integer)(curIdxs.get(i));
						String curAttr[]=((String)(attributeList.get(curIdx.intValue()))).split("_");
						newAttr.add(curAttr[0]+"_"+curKey);
				}
			} else {
				//If this feature should be compressed
				newAttr.add(hashName+"_"+curKey);				
			}
		}
		return newAttr;
	}
	
	
	/**
	 * Parse the .arff file to read in the attributes, data, and mutant names
	 * @param arffFileName
	 */
	private void parseArffFile(String arffFileName){
		attributeList = new ArrayList();

		try{
			FileReader inFile = new FileReader(arffFileName);
			BufferedReader arffReader = new BufferedReader(inFile);

			//Echo everthing before the @attribute
			String line = arffReader.readLine();
			while (!((line.toLowerCase()).startsWith("@attribute"))){
				System.out.println(line);
				line = arffReader.readLine();
			}

			// Read in all attributes
			while (!((line.toLowerCase()).startsWith("@data"))){
				String attributeLine[]=line.split(" ");
				attributeList.add(attributeLine[1]);
				line = arffReader.readLine();
			}
						
			arffReader.close();
		} catch (Exception e){
			e.printStackTrace();
		}		
		return;
	}
	
	/**
	 * Read in the list of residues to use
	 * @param residueListName
	 */
	private ArrayList readResidueList(String residueListName){
		ArrayList featureList=new ArrayList();
		try{
			FileReader inFile = new FileReader(residueListName);
			BufferedReader listReader = new BufferedReader(inFile);
			
			String line = listReader.readLine();
			while (line!=null){
				if (!line.startsWith("#")){
					String cutString[]=line.split("-");
					int startIdx=Integer.parseInt(cutString[0]);
					int stopIdx=Integer.parseInt(cutString[1]);
					for (int i=startIdx; i<=stopIdx; i++){
						featureList.add(new Integer(i));
					}
				}
				line = listReader.readLine();
			}
			listReader.close();			
			
		} catch (Exception e){
			e.printStackTrace();
		}		
		return featureList;
	}
	
	/**
	 * Provide command line interface
	 * @param args
	 */	
	public static void main (String args []){
		if (args.length != 2){
			System.out.println("Combine all 2d residues that are not in the residueList");
			System.out.println("\tModify2dResolution <residueList> <arffName>");
			System.out.println("ex: Modify2dResolution highResRegions.txt K3.2d.arff");
		} else if (args.length == 2){
			Modify2dResolution myModify2dResolution = new Modify2dResolution(args[0], args[1]);			
		} 
	}	
}

