/*
 * Created on June 9th, 2006
 *
 * A program to normalize attributes in an arff file. It produces a new file in an arff
 * format with normalized attributes.
 * 
 * @version	1.0 9 Jan 2006
 * @author 	Sam Danziger
 * 
 * @version .9 9 Jan 2006
 * @author  Vadim Bichutskiy
 */

/*
File: NormalizeArff.java
Date      Author       			Changes
Jan 9  06 Vadim Bichutskiy      Created as NormalizeAttributes.java and ArffInstance.java
Jan 9  06 Sam Danziger		    Changed name and added commandline parsing wrapper and inner class
*/

import java.io.*;
import java.util.*;
import java.text.DecimalFormat;

public class NormalizeArff
{
	public class ArffInstance
	{
	   public String comment;
	   public Vector values;

	   public ArffInstance(String str, Vector v)
	   {
		   comment = str;
		   values = v;
	   }

	   public ArffInstance(String str)
	   {
		   comment = str;
		   values = new Vector();
	   }
	}
	
	private DecimalFormat form; //Use this to limit number output to #.#### 
	
	public NormalizeArff(String inFile, String outFile)
	{
		form = new DecimalFormat();
		form.applyPattern("#.#####");
		
		BufferedReader input = null;
		FileOutputStream output = null;
		try
		{
		   	FileReader file = new FileReader(inFile);
		    input = new BufferedReader(file);
			output = new FileOutputStream(outFile);
		}
		catch(FileNotFoundException fnfe)
		{
			System.out.println(fnfe);
		}

		PrintWriter pw = new PrintWriter(output, true);
		String line="";
		double attributeValue;
		double maxValue = Double.NEGATIVE_INFINITY;
		Vector arffInstances = new Vector();
		String comment="";

		if(inFile.indexOf("3d") != -1)
			normalize3d(input, pw);
		else
		{
			try
			{
				while( (line = input.readLine()) != null)
				{
					if(line.charAt(0) != '%' && line.charAt(0) != '@')
					{
						String [] tokens = line.split(",");
						ArffInstance ai = new ArffInstance(comment);
						for(int i=0; i<tokens.length; i++)
						{
							if(tokens[i].equals("?"))
								ai.values.addElement(tokens[i]);
							else
							{
								attributeValue = Double.parseDouble(tokens[i]);
								if(attributeValue > maxValue)
									maxValue = attributeValue;
								ai.values.addElement(new Double(attributeValue));
							}

						}

						arffInstances.addElement(ai);
					}
					else if(line.charAt(0) == '@')
						pw.println(line);
					else if(line.charAt(0) == '%')
						comment = line;
				}

				System.out.println("The max value is " + maxValue);

				for(int i=0; i<arffInstances.size(); i++)
				{
					ArffInstance ai1 = (ArffInstance)arffInstances.elementAt(i);
					pw.println(ai1.comment);
					Vector values = ai1.values;

					for(int j=0; j<values.size(); j++)
					{
						if(values.elementAt(j) instanceof Double)
						{
							double value = ((Double)values.elementAt(j)).doubleValue();
							double result = value/maxValue;

							if( Math.floor(result) == result)
							{
								int result1 = (int)result;
								pw.print(form.format(result1));
								pw.print(",");
							}
							else
							{
								pw.print(form.format(result));
								pw.print(",");
							}
						}
						else
						{
							String str = (String)values.elementAt(j);
							pw.print(str);
							pw.print(",");
						}
					}

					pw.println();
				}
			}
			catch(IOException ioe)
			{
				System.out.println(ioe);
			}
		}
	}

	private void normalize3d(BufferedReader input, PrintWriter pw)
	{
		String line="";
		double attributeValue;
		double maxValue = Double.NEGATIVE_INFINITY;
		Vector arffInstances = new Vector();
		String comment="";
		double attributeValue2=0;
		double maxValue2 = Double.NEGATIVE_INFINITY;

		try
		{
			while( (line = input.readLine()) != null)
			{
				if(line.charAt(0) != '%' && line.charAt(0) != '@')
				{
					String [] tokens = line.split(",");
					ArffInstance ai = new ArffInstance(comment);
					for(int i=0; i<tokens.length; i++)
					{
						if(i>=582 && i<590)
						{
							if(tokens[i].equals("?"))
								ai.values.addElement(tokens[i]);
							else
							{
								attributeValue2 = Double.parseDouble(tokens[i]);
								if(attributeValue2 > maxValue2)
									maxValue2 = attributeValue2;
								ai.values.addElement(new Double(attributeValue2));
							}
						} else if(tokens[i].equals("?")) {
							ai.values.addElement(tokens[i]);
						} else {
							attributeValue = Double.parseDouble(tokens[i]);
							if(attributeValue > maxValue) {
								maxValue = attributeValue;
							}
							ai.values.addElement(new Double(attributeValue));
						}
					}

					arffInstances.addElement(ai);
				}
				else if(line.charAt(0) == '@')
					pw.println(line);
				else if(line.charAt(0) == '%')
					comment = line;
			}

			System.out.println("The max value is " + maxValue);
			System.out.println("The max value 2 is " + maxValue2);

			for(int i=0; i<arffInstances.size(); i++)
			{
				ArffInstance ai1 = (ArffInstance)arffInstances.elementAt(i);
				pw.println(ai1.comment);
				Vector values = ai1.values;
				double result;

				for(int j=0; j<values.size(); j++)
				{
					if(values.elementAt(j) instanceof Double)
					{
						double value = ((Double)values.elementAt(j)).doubleValue();

						if(j>=582 && j<590)
							result = value/maxValue2;
						else
							result = value/maxValue;
                        

						if( Math.floor(result) == result)
						{
							int result1 = (int)result;
							pw.print(form.format(result1));
							pw.print(",");
						}
						else
						{
							pw.print(form.format(result));
							pw.print(",");
						}
					}
					else
					{
					 	String str = (String)values.elementAt(j);
						pw.print(str);
						pw.print(",");
					}
				}

				pw.println();
			}
		}
		catch(IOException ioe)
		{
			System.out.println(ioe);
		}
	}
	
	public static void main (String args []) {
		if ((args.length != 2)){
			System.out.println("Normalize the .arff file" );
			System.out.println("NormalizeArff <inFile> <outFile>" );
			System.out.println("NormalizeArff 2d.arff 2d.norm.arff" );
		} else {
			String inFile = args[0];
			String outFile = args[1];
			NormalizeArff myNormalizeArff = new NormalizeArff(inFile,outFile);
		}
	}
}
