import java.io.*;
import java.util.*;

/**
 * 
 * @author Sam Danziger
 *
 * Reorder a arff file and add the question marks to missing values that 
 * exist in a .def file
 */
class PadArffFromDef {
	ArrayList reflist; //The list of mutants from the reference list
	ArrayList refdata; //Used to determine the class of a mutant
	ArrayList mutNames; //The list of names from the arff file to pad
	ArrayList mutData;  //The list of data associated with the mutNames
	
	public PadArffFromDef(String refArff, String arffName, boolean addClass){
		reflist = new ArrayList();
		refdata = new ArrayList();
		mutNames = new ArrayList();
		mutData = new ArrayList();
		int attribCount=0; //The attribute count for the .arff file
		
		try {
			//Create the list of all the names and activity from the .def file
			FileReader inFile = new FileReader(refArff);
			BufferedReader in = new BufferedReader(inFile);			

			String line = in.readLine();
			while (line != null){
				if (line.length() != 0) {
					StringTokenizer tok = new StringTokenizer(line);
					reflist.add("%"+(tok.nextToken()).toLowerCase());				
					refdata.add((tok.nextToken()).toLowerCase());
				}
				line = in.readLine();		
			}			
			
			in.close();
			
			//Read in the names and data from the arff File
			inFile = new FileReader(arffName);
			in = new BufferedReader(inFile);
			
			line = in.readLine();
			while (!((line.toLowerCase()).startsWith("@data"))){
				if ((line.toLowerCase()).startsWith("@attribute")){
					attribCount++;
				}
				System.out.println(line);
				line = in.readLine();
			}
			System.out.println(line);
			
			line = in.readLine();
			while (line != null){
				mutNames.add(line.toLowerCase());				
				line = in.readLine();
				mutData.add(line.toLowerCase());
				line = in.readLine();		
			}	
			
			in.close();
			
			//Print out results.
			for (int i=0; i<reflist.size(); i++){
				String refMut = (String)(reflist.get(i));
				int idx = mutNames.indexOf(refMut);
				if (idx == -1) {
					System.out.println(refMut);
					for (int j=0; j<(attribCount-1); j++){
						System.out.print("?,");
					}
					//Print the class
					String curRef = (String)refdata.get(i);
 					
 					StringTokenizer tok = new StringTokenizer(curRef,",");
					String curTok=null;
					while (tok.hasMoreTokens()){
						curTok=tok.nextToken();
					}
					if (addClass == true) {
						System.out.println(curTok+",");
					} else {
						System.out.println("?,");			
					}
				} else {
					System.out.println(mutNames.get(idx));
					System.out.println(mutData.get(idx));
				}
			}
			
		} catch (Exception e){
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}
	
	/**
	 * Reorder a arff file and add the question marks to missing values that 
	 * exist in a .def file
	 *
	 * @param args
	 */
	public static void main (String args []) {
		if ((args.length != 2) && (args.length != 3)){
			System.out.println("Reorder a arff file and add the question marks to missing values that exist in a .def file");
			System.out.println("PadArffFromDef <defFile> <arffName> <noActivity - optional>" );
			System.out.println("\tnoActivity:  Include this paramater to not add an activity to the padded lines" );
			System.out.println("PadArffFromDef K2.def 1d.arff > 1d.full.arff" );
		} else {
			String defFile = args[0];
			String arffName = args[1];
			boolean addClass;
			if (args.length == 2){
				addClass=true;
			} else {
				addClass=false;
			}
			
			PadArffFromDef myPadArffFromDef = new PadArffFromDef(defFile,arffName,addClass);
		}
	}

}