import java.io.*;
import java.util.*;

/**
 * @author Sam Danziger
 *
 * The opposite of RemoveListFromArff.  Output an ARFF file that contains only the mutants in
 * input .list or .def file
 */
public class PreserveListFromArff {
	ArrayList mutsToPreserve;
	/**
	 * Preserve all of the mutants listed in a input list (in .def or .list format)
     * from a .arff files
	 * @param arffName
	 * @param listName
	 * @param outName
	 * @param s1d	Set to true to filter attributes for the s1d set
	 */
	public PreserveListFromArff (String arffName, String listName, String outName, boolean s1d){
		mutsToPreserve = readList(listName);
		try {
			FileReader inFile = new FileReader(arffName);
			BufferedReader in = new BufferedReader(inFile);
			
			PrintWriter out = new PrintWriter( new FileWriter(outName));
			PrintWriter out2 = new PrintWriter( new FileWriter("excluded.txt"));
			
			String line = in.readLine();
			
			int idx=0;
			ArrayList remIdxList=new ArrayList();
			while(!(line.toLowerCase()).startsWith("@data")){
				if (s1d == false){
					out.println(line);
					line = in.readLine();
				} else {
					if ((line.toLowerCase()).startsWith("@attribute")){
						if(mutInList2(line, mutsToPreserve)){
							out.println(line);
						} else {
							remIdxList.add(new Integer(idx));
						}
						idx++;
					} else {
						out.println(line);
					}
					line = in.readLine();
				}
			}
			out.println(line);
			
			String commentLine = in.readLine();
			String dataLine = in.readLine(); 
			while(commentLine != null){
				if (!mutInList(commentLine, mutsToPreserve)){
					out2.println(commentLine);
				} else {
					out.println(commentLine);
					if (s1d == false) {
						out.println(dataLine);
					} else {
						//Cut elements out of the list
						StringTokenizer tok = new StringTokenizer(dataLine,",");
						int ctr=0;
						while (tok.hasMoreTokens()){
							String curTok=tok.nextToken();
							if (!remIdxList.contains(new Integer(ctr))){
								out.print(curTok+",");
							}
							ctr++;
						}
						out.print("\n");
					}
				}
				
				commentLine = in.readLine();
				dataLine = in.readLine(); 
			}
			out.close();
			out2.close();
			
		} catch (Exception e){
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}

	
	/**
	 * Read the contents of listname into the ArrayList
	 * @param listName
	 * @return
	 */
	private ArrayList readList (String listName){
		ArrayList mutList = new ArrayList();
		try {
			FileReader inFile = new FileReader(listName);
			BufferedReader in = new BufferedReader(inFile);
			
			String line = in.readLine();
			while (line != null){
				StringTokenizer tok = new StringTokenizer(line);
				if (tok.hasMoreTokens()){
					mutList.add((tok.nextToken()).toLowerCase());
				}
				line = in.readLine();
			}
			in.close();
			
		} catch (Exception e){
			System.out.println(e.getMessage());
			e.printStackTrace();
		}		
		return mutList;
	}
	
	/**
	 * Return true if the line contains exactly a mutant in the list
	 * @param line
	 * @return
	 */
	private boolean mutInList(String line, ArrayList list){
		String subLine = line.substring(1,line.length());
		StringTokenizer tok = new StringTokenizer(subLine);
		String curTok = tok.nextToken();
		
		return list.contains(curTok.toLowerCase());
	}
	
	/**
	 * Used to parse attribute lines
	 * @param line
	 * @param list
	 * @return
	 */
	private boolean mutInList2(String line, ArrayList list){
		
		StringTokenizer tok = new StringTokenizer(line);
		String curTok = tok.nextToken();
		curTok = tok.nextToken();
		
		return list.contains(curTok.toLowerCase());
	}
	
	/**
	 * Parse a comment line to see if has exactly the mutant
	 * @param line
	 * @param mutant
	 * @return
	 */
	private boolean hasMutant(String line, String mutant){
		StringTokenizer tok = new StringTokenizer(line);
		boolean hasMut = false;
		while (tok.hasMoreTokens()){
			String curTok = tok.nextToken();
			if (curTok.equalsIgnoreCase(mutant)){
				hasMut = true;
				break;
			}
		}
		return hasMut;
	}
	
	/**
	 * Preserve only the mutants listed in a input list (in .def or .list format)
     * from a .arff files.  Print result to <stdout>
	 * PreserveListFromArff <arffName> <listName> <outName>
	 * PreserveListFromArff in.arff BaroniTable3_implied_inactive.def noImp.in.arff
	 *
	 * @param args
	 */
	public static void main (String args []) {
		if (args.length != 3 && args.length != 4){
			System.out.println("Preserve only the mutants listed in a input list (in .def or .list format) from a .arff files.");
			System.out.println("\tSet the s1d flag to also process attributes (for the s1d set)");
			System.out.println("PreserveListFromArff <arffName> <listName> <outName> <s1d -- optional>" );
			System.out.println("PreserveListFromArff all.arff X1.def X1.arff" );
		} else {
			String arffName = args[0];
			String listName = args[1];
			String outName = args[2];
			if (args.length==3){
				PreserveListFromArff myPreserveListFromArff = new PreserveListFromArff(arffName, listName, outName, false);
			} else {
				PreserveListFromArff myPreserveListFromArff = new PreserveListFromArff(arffName, listName, outName, true);
			}
			//System.out.println("Completed Execution: RemoveListFromArff "+arffName+" "+listName);
		}
	}
}