import java.io.BufferedReader;
import java.io.FileReader;
import java.util.ArrayList;

/**
 * @author Sam Danziger
 *
 * Randomize the order of the instances in a .arff file and spit the result to stdout
 */
public class RandomizeArff {
	private ArrayList mutNames;
	private ArrayList data;
	
	public RandomizeArff (String inArff){
		mutNames = new ArrayList();
		data = new ArrayList();
		
		try {
			FileReader inFile = new FileReader(inArff);
			BufferedReader in = new BufferedReader(inFile);

			//Title
			String line = in.readLine();
			while (!(line.toLowerCase()).startsWith("@data")){
				System.out.println(line);
				line = in.readLine();
			}
			System.out.println(line);

			//@data : Store the names and data
			line = in.readLine();
			while (line != null){
				//Name Line
				if (line.startsWith("%")){
					mutNames.add(line);
				}
				line = in.readLine();

				//Data line
				data.add(line);
				line = in.readLine();
			}
			in.close();
			
			//Spit the lines back in random order.
			int numData=data.size();
			int numNames=mutNames.size();
			boolean useNames=false;
			if (numData == numNames) {
				useNames=true;
			} 
			
			while(numData > 0){
				int curIdx = (int)(Math.floor(Math.random()*numData));
				//System.err.println(curIdx);
				if(useNames==true){
					System.out.println((String)(mutNames.get(curIdx)));
					mutNames.remove(curIdx);
				}				
				System.out.println((String)(data.get(curIdx)));
				data.remove(curIdx);
				
				numData=data.size();
			}			
		} catch (Exception e){
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}
	
	/**
	 * Randomize the data in a .arff file.
	 * RandomizeArff <arffName> > <newArff>
	 *
	 * @param args
	 */
	public static void main (String args []) {
		if (args.length != 1){
			System.out.println("Randomize the data in a .arff file.");
			System.out.println("RandomizeArff <arffName> > <newArff>" );
		} else {
			String arffName = args[0];
			RandomizeArff myRandomizeArff = new RandomizeArff(arffName);
		}
	}
}
