import java.io.BufferedReader;
import java.io.FileReader;
import java.util.ArrayList;

/*
 * Created on Jan 10, 2006
 *
 * Read in a numerical only arff file with %name comments before each data line.
 * Store this .arff file for internal manipulation. 
 */

/**
 * @author sdanzige
 *
 * Read in a numerical only arff file with %name comments before each data line.
 * Store this .arff file for internal manipulation.
 */
public class ReadArff {
    
	private String title;
	private ArrayList attributes;
	private String dataline;
	private ArrayList datanames;
	private double [][] data;
	
	/**
	 * Read in the Arff file 
	 * @param fileName
	 */
	public ReadArff (String fileName){
		try {
			FileReader inFile = new FileReader(fileName);
			BufferedReader in = new BufferedReader(inFile);
			
			String line=in.readLine();
			if (line.toLowerCase().startsWith("@relation")){
				title=new String(line);
			} else {
				throw new Exception(fileName+" - BAD ARFF FILE: Does not start with @relation");				
			}
			
			line=in.readLine();
			attributes = new ArrayList();
			while (line.toLowerCase().startsWith("@attribute")) {
				attributes.add(line);
				line=in.readLine();
			}
			
			if (line.toLowerCase().startsWith("@data")){
				dataline=new String(line);
			} else {
				throw new Exception(fileName+" - BAD ARFF FILE: Does not start with @data");
			}
			
			line=in.readLine();		
			int numData=0;
			ArrayList dataLines = new ArrayList();
			datanames = new ArrayList();
			while (line!=null) {
				if (line.startsWith("%")) {
					datanames.add(line);
				} else {
					String [] curLine=line.split(",");
					Object [] curData = new Object[curLine.length];
					for (int i=0; i<curLine.length; i++){
						curData[i]=curLine[i];
					}
					dataLines.add(curData);
					numData++;
				}
				
				line=in.readLine();
			}
			
			//Convert dataLines into data
			data = new double[numData][attributes.size()];
			for (int dataIdx=0; dataIdx < numData; dataIdx++){
				Object [] curDataLine = (Object [])(dataLines.get(dataIdx));

				if (curDataLine.length != attributes.size()){
					throw new Exception(fileName+" - BAD ARFF FILE: data line "+dataIdx+"(base 0) has "+curDataLine.length+" attributes");
				}
				for (int attrIdx=0; attrIdx < attributes.size(); attrIdx++){
					if ( ((String)(curDataLine[attrIdx])).startsWith("?") ) {
						data[dataIdx][attrIdx]=Double.NaN;
					} else {
						double curval=Double.parseDouble((String)curDataLine[attrIdx]);
					    data[dataIdx][attrIdx]=curval;
					}
				}				
			}
			
		} catch (Exception e){
			System.out.println(e.getMessage());
			e.printStackTrace();
		}		
	}
	
	/**
	 * Return the name of the data at <idx>
	 * @param idx
	 * @return
	 */
	public String getInstanceName(int idx){
		return ((String)(datanames.get(idx)));
	}
	
	public int getNumInstances(){
		return (datanames.size());
	}

	/**
	 * Return the @relation line
	 * @return
	 */
	public String getTitle(){
		return this.title;
	}
	
	/**
	 * Return the list of String names in an arrayList
	 * @return
	 */
	public ArrayList getAttributes(){
		return this.attributes;
	}
	
	/**
	 * Return the data line.  Included so that case is not changed
	 * @return
	 */
	public String getDataLine(){
		return this.dataline;
	}
	
	/**
	 * Return the data at instance <idx>
	 * @param args
	 */
	public double [] getInstanceData(int idx){
		return data[idx];
	}
	
	/**
	 * Return the data at attribute <idx>.  Replace NaN with the mean value
	 * @param idx
	 * @return
	 */
	public double [] getAttributeData(int idx){
		double [] attributeData = new double [data.length];
		
		double sum=0;
		int count=0;
		for (int i=0; i<data.length; i++){
			attributeData[i]=data[i][idx];
			if ( !Double.isNaN(attributeData[i]) ){
				count++;
				sum=sum+attributeData[i];
			} 
		}
		
		double mean=sum/count;
		for (int i=0; i<data.length; i++){
			if ( Double.isNaN(attributeData[i]) ){
				attributeData[i]=mean;
			} 
		}
				
		return attributeData;
	}
	
	/**
	 * Used for testing 
	 * @param args
	 */
	public static void main(String[] args) {
		ReadArff myReadArff = new ReadArff(args[0]);
		
		double [] attr1 = myReadArff.getAttributeData(1);
		
		double sum=0;
		for (int i=0; i < attr1.length; i++){
			sum=sum+attr1[i];
			System.out.println(attr1[i]);
		}
		System.out.println("sum: "+sum);
	}
}
