import java.io.*;
import java.util.StringTokenizer;
/**
 *
 * @author sdanzige
 *
 * Read in a given result file and provide and interface to it's contents
 */
class ReadResult{
		String pred;
		String truth;

		/**
		 * Read in the .result file
		 * @param fileName
		 */
		public ReadResult(String fileName){
				pred = null;
				truth = null;
				String line = new String();
				try{
						FileReader inFile = new FileReader(fileName);
						BufferedReader in = new BufferedReader(inFile);

						line = in.readLine();
						if (line != null) {
								StringTokenizer tok = new StringTokenizer(line);
								String curTok = tok.nextToken();
								pred = tok.nextToken();

								line = in.readLine();
								tok = new StringTokenizer(line);
								curTok = tok.nextToken();
								truth = tok.nextToken();
						}
						in.close();
				}catch (Exception e){
						System.err.println("line = "+line);
						System.err.println(e.getMessage());
						//e.printStackTrace();
				}
		}

		/**
		 * Return the predicted classification
		 * @return
		 */
		public String getPred(){
				return pred;
		}

		/**
		 * Return the true classification
		 * @return
		 */
		public String getTruth(){
				return truth;
		}

		/**
		 * Return TP,FP,FN,TN
		 * @param positive : The name of the positive value. EX: active
		 * @return
		 */
		public String getConfusion(String positive){
				if(pred == null){
						return null;
				}

				if (pred.equalsIgnoreCase(truth)){
						if (pred.equalsIgnoreCase(positive)){
								return "TP";
						} else {
								return "TN";
						}
				} else {
						if (pred.equalsIgnoreCase(positive)){
								return "FP";
						} else {
								return "FN";
						}
				}
		}

		/**
		 * Testing and such
		 * arg[0] = fileName
		 * arg[1] = positive value
		 * @param args
		 */
		public static void main (String args []) {
				if (args.length != 2){
						System.out.println("Read in a result file and display the contents" );
						System.out.println("ReadResult <filename> <positive>" );
						System.out.println("Ex: ReadResult r273l.result active" );
				} else {
						String fileName = args[0];
						String positive = args[1];
						ReadResult myResult = new ReadResult(fileName);
						System.out.println("PRED = "+myResult.getPred());
						System.out.println("TRUTH = "+myResult.getTruth());
						System.out.println("CONFUSION = "+myResult.getConfusion("positive"));
				}


		}

}
