import java.io.*;
import java.util.*;

/**
 *
 * @author Sam Danziger
 *
 * Remove the extra info from the 3d arff file and save
 * it as a separate file
 */
class RemoveActive {
	public RemoveActive(String arffName){
		try {
			FileReader inFile = new FileReader(arffName);
			BufferedReader in = new BufferedReader(inFile);

			String outName = addPrefix(arffName,"noActivity");
			PrintWriter out = new PrintWriter( new FileWriter(outName));

			//Title
			String line = in.readLine();
			while (!(line.toLowerCase()).startsWith("@attribute")){
				out.println(line);
				line = in.readLine();
			}

			//Attributes
			ArrayList extraIDX = new ArrayList();
			int idx = 0;
			int classIdx = -1; //The index of active,inactive
			while (!(line.toLowerCase()).startsWith("@data")){

				if ((line.toLowerCase()).indexOf("active")!=-1){
					classIdx = idx;
				} else {
					out.println(line);
				}
				line = in.readLine();
				idx++;
			}

			//@data
			out.println(line);

			line = in.readLine();
			while (line != null){
				//Name Line
				out.println(line);
				line = in.readLine();

				//Data line
				StringTokenizer tok = new StringTokenizer(line,",");
				idx = 0;
				while(tok.hasMoreTokens()){
					String curTok = tok.nextToken();
					if (idx != classIdx){
						out.print(curTok+",");
					}
					idx++;
				}

				out.println();
				line = in.readLine();
			}

			out.close();
			in.close();
		} catch (Exception e){
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	 * Return <dir>/prefix.<fileName>
	 * @param fileName
	 * @param prefix
	 * @return
	 */
	private String addPrefix (String fileName, String prefix){
		String newName = null;

		int idx = fileName.lastIndexOf("/");
		if (idx == -1){
			newName = prefix+"."+fileName;
		} else {
			newName = fileName.subSequence(0,idx+1)+prefix+"."+fileName.subSequence(idx+1,fileName.length());
		}
		return newName;
	}

	/**
	 * Remove The activity (active, inactive) from a .arff file
	 * RemoveActive <arffName>
	 * RemoveActive 3d.arff
	 *
	 * @param args
	 */
	public static void main (String args []) {
		if (args.length != 1){
			System.out.println("Remove the activity (active, inactive) from a .arff file");
			System.out.println("RemoveActive <arffName>" );
			System.out.println("RemoveActive 3d.arff" );
		} else {
			String arffName = args[0];
			RemoveActive myRemoveActive = new RemoveActive(arffName);
			System.out.println("Completed Execution: RemoveActive "+arffName);
		}
	}

}
