import java.io.*;
import java.util.*;

/**
 * 
 * @author Sam Danziger
 *
 * Remove any line of a composite .arff file that contains a value other
 * than active or inactive.  Put the excluded in a .excl2 file
 */
class RemoveNotActiveInactive{
	ArrayList excluded;
	/**
	 * Constructor
	 * @param fileName
	 */
	public RemoveNotActiveInactive(String fileName){
		excluded = new ArrayList();
		try {
			FileReader inFile = new FileReader(fileName);
			BufferedReader in = new BufferedReader(inFile);
			
			String outName = addPrefix(fileName,"remNull");
			PrintWriter out = new PrintWriter( new FileWriter(outName));
			
			String line = in.readLine();
			//Header
			while (!(line.toLowerCase()).startsWith("@data")){
				out.println(line);
				line = in.readLine();
			}
			out.println(line);
			
			//TData
			String commentLine = in.readLine();
			String dataLine = in.readLine();

			int lineNum = 0;
			while (commentLine != null){
				
				boolean exclude = true;
				StringTokenizer tok = new StringTokenizer(dataLine,",");
				//if(commentLine.equals("%e198a")){
				//	System.out.println("Here");
				//}

				while (tok.hasMoreTokens()){
					String curTok = tok.nextToken();
					
					if (curTok.equalsIgnoreCase("active") || curTok.equalsIgnoreCase("inactive") || curTok.equalsIgnoreCase("?")){
						exclude = false;
					} else {
						exclude = true;
						break;
					}
				}
				
				if (exclude == false){
					out.println(commentLine);
					out.println(dataLine);
				} else {
					excluded.add(commentLine);
				}
				
				commentLine = in.readLine();
				dataLine = in.readLine();
				lineNum++;
			}
			
			in.close();
			out.close();
			writeList(excluded,fileName+".excl2");
						
		} catch (Exception e){
			System.out.println(e.getMessage());
			e.printStackTrace();
		}		

	}
	
	/**
	 * Write all elements of any collection to a file
	 * @param list
	 * @param fileName
	 */
	private void writeList(Collection list, String fileName){
		try{
			PrintWriter output = new PrintWriter( new FileWriter(fileName));
			Object [] objList = list.toArray();
			for(int i=0;i<objList.length;i++){
				output.println(objList[i]);
			}
			output.close();
		}catch (Exception e){
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}
	
	/**
	 * Return <dir>/prefix.<fileName>
	 * @param fileName
	 * @param prefix
	 * @return
	 */
	private String addPrefix (String fileName, String prefix){
		String newName = null;
		
		int idx = fileName.lastIndexOf("/");
		if (idx == -1){
			newName = prefix+"."+fileName;
		} else {
			newName = fileName.subSequence(0,idx+1)+prefix+"."+fileName.subSequence(idx+1,fileName.length());
		}
		return newName;
	}
	
	/**
	 * Remove any lines that aren't all active or inactive
	 * RemoveNotActiveInactive <fileName>
	 * RemoveNotActiveInactive fileName
	 *
	 * @param args
	 */
	public static void main (String args []) {
		if ((args.length != 1)){
			System.out.println("Remove any lines that aren't all active or inactive or ?" );
			System.out.println("RemoveNotActiveInactive <fileName>" );
			System.out.println("RemoveNotActiveInactive fileName" );
		} else {
			String arffName = args[0];
			RemoveNotActiveInactive myRemoveNotActiveInactive = new RemoveNotActiveInactive(arffName);
			System.out.println("Completed Execution: RemoveNotActiveInactive "+arffName);
		}

	}
}