import java.text.DecimalFormat;
import java.util.ArrayList;

/*
 * Created on Jan 10, 2006
 *
 * Replace the numberical values in a .arff file with their Z values
 */

/**
 * @author sdanzige
 *
 * Replace the numberical values in a .arff file with their Z values
 */


public class ScaleArff {

	/**
	 * Copied From: http://www.faqs.org/docs/javap/c5/ex-5-2-answer.html
	 * @author David J. Eck  (eck@hws.edu)
	 *
	 * An object of class StatCalc can be used to compute several simple statistics
	 * for a set of numbers.  Numbers are entered into the dataset using
	 * the enter(double) method.  Methods are provided to return the following
	 * statistics for the set of numbers that have been entered: The number
	 * of items, the sum of the items, the average, and the standard deviation.
	 */
	public class StatCalc {

	   private int count;   // Number of numbers that have been entered.
	   private double sum;  // The sum of all the items that have been entered.
	   private double squareSum;  // The sum of the squares of all the items.

	   public void enter(double num) {
	         // Add the number to the dataset.
	      count++;
	      sum += num;
	      squareSum += num*num;
	   }

	   public int getCount() {   
	         // Return number of items that have been entered.
	      return count;
	   }

	   public double getSum() {
	         // Return the sum of all the items that have been entered.
	      return sum;
	   }

	   public double getMean() {
	         // Return average of all the items that have been entered.
	         // Value is Double.NaN if count == 0.
	      return sum / count;  
	   }

	   public double getStandardDeviation() {  
	        // Return standard deviation of all the items that have been entered.
	        // Value will be Double.NaN if count == 0.
	      double mean = getMean();
	      return Math.sqrt( squareSum/count - mean*mean );
	   }

	}  // end of class StatCalc
	
	/**
	 * Rescale a .arff file by replacing double values with their Z value 
	 * @param fileName
	 */
	private DecimalFormat form;
	
	public ScaleArff (String fileName) {
		form = new DecimalFormat();
		form.applyPattern("#.############");
		
		ReadArff arffFile = new ReadArff (fileName);
		StatCalc myStatCalc = new StatCalc ();
		
		//Echo the header.
		System.out.println(arffFile.getTitle());
		
		//Echo the attributes.
		ArrayList attributes = arffFile.getAttributes();
		for (int i=0; i<attributes.size(); i++) {
			System.out.println(attributes.get(i));
		}
		
		//Echo data
		System.out.println(arffFile.getDataLine());
		
		//Calculate the overall standard deviation.
		for (int i=0; i<arffFile.getNumInstances(); i++){
			double [] curData=arffFile.getInstanceData(i);
			
			for (int j=0; j<curData.length; j++) {
				if ( !Double.isNaN(curData[j]) ){
					myStatCalc.enter(curData[j]);
				}
			}
		}
		
		//Echo each instance line
		for (int i=0; i<arffFile.getNumInstances(); i++){
			System.out.println(arffFile.getInstanceName(i));
			double [] curInstanceData=arffFile.getInstanceData(i);
			for (int j=0; j<curInstanceData.length; j++){
				double curData=curInstanceData[j];
				
				if ( Double.isNaN(curData) ) {
					System.out.print("?,");
				} else {
					curData = (myStatCalc.getMean()-curData)/myStatCalc.getStandardDeviation();
					System.out.print(form.format(curData)+",");
				}
			}
			System.out.print("\n");
		}
	}
	
	public static void main(String[] args) {
		if ((args.length != 1)){
			System.out.println("Replace all double values in a .arff with their Z value." );
			System.out.println("ScaleArff <arffName>" );
			System.out.println("ScaleArff train.2d.arff" );
		} else {
			String arffName = args[0];
			ScaleArff myScaleArff = new ScaleArff(arffName);
		}
	}
}
