import java.io.*;
import java.util.*;

/**
 * 
 * @author Sam Danziger
 *
 * Remove the 1d info from the 3d arff file and save 
 * it as a separate file
 */
class Separate1Dfrom3D {
	public Separate1Dfrom3D(String arffName){
		try {
			FileReader inFile = new FileReader(arffName);
			BufferedReader in = new BufferedReader(inFile);
			
			String outName = addPrefix(arffName,"no1d");
			PrintWriter out = new PrintWriter( new FileWriter(outName));

			String extraName = "1d.arff";
			PrintWriter extraOut = new PrintWriter( new FileWriter(extraName));
			
			//Title
			String line = in.readLine();			
			while (!(line.toLowerCase()).startsWith("@attribute")){
				out.println(line);
				extraOut.println(line);
				line = in.readLine();
			}
			
			//Attributes
			ArrayList extraIDX = new ArrayList();
			int idx = 0;
			int classIdx = -1; //The index of active,inactive
			while (!(line.toLowerCase()).startsWith("@data")){
				
				if ((line.toLowerCase()).indexOf("class")!=-1){
					out.println(line);
					extraOut.println(line);
					classIdx = idx;
				} else if((line.indexOf("{"))==-1){
					out.println(line);
				} else {
					extraOut.println(line);
					extraIDX.add(new Integer(idx));
				}
				line = in.readLine();
				idx++;
			}
			
			//@data
			out.println(line);
			extraOut.println(line);
			
			line = in.readLine();			
			while (line != null){
				//Name Line
				out.println(line);
				extraOut.println(line);					
				line = in.readLine();				
				
				//Data line
				StringTokenizer tok = new StringTokenizer(line,",");
				idx = 0;
				while(tok.hasMoreTokens()){
					String curTok = tok.nextToken();
					if (idx == classIdx){
						extraOut.print(curTok+",");																
						out.print(curTok+",");						
					} else if (extraIDX.contains(new Integer(idx))){
						extraOut.print(curTok+",");																
					} else {
						out.print(curTok+",");
					}
					idx++;
				}
				
				out.println();
				extraOut.println();
				
				line = in.readLine();				
			}	
			
			out.close();
			extraOut.close();
			in.close();		
		} catch (Exception e){
			System.out.println(e.getMessage());
			e.printStackTrace();
		}			
	}
	
	/**
	 * Return <dir>/prefix.<fileName>
	 * @param fileName
	 * @param prefix
	 * @return
	 */
	private String addPrefix (String fileName, String prefix){
		String newName = null;
		
		int idx = fileName.lastIndexOf("/");
		if (idx == -1){
			newName = prefix+"."+fileName;
		} else {
			newName = fileName.subSequence(0,idx+1)+prefix+"."+fileName.subSequence(idx+1,fileName.length());
		}
		return newName;
	}
	
	/**
	 * Remove the 1d info from the 3d arff file and save 
	 * it as a separate file
	 * Separate1Dfrom3D <arffName>
	 * Separate1Dfrom3D 3d.arff
	 * 
	 * @param args
	 */
	public static void main (String args []) {
		if (args.length != 1){
			System.out.println("Remove the 1d info from the 3d arff file and save it as a separate file");
			System.out.println("Separate1Dfrom3D <arffName>" );
			System.out.println("Separate1Dfrom3D 3d.arff" );
		} else {
			String arffName = args[0];
			Separate1Dfrom3D mySeparate1Dfrom3D = new Separate1Dfrom3D(arffName);
			System.out.println("Completed Execution: Separate1Dfrom3D "+arffName);
		}
	}

}
