import java.io.*;
import java.util.*;
/**
 *
 * @author sdanzige
 *
 * Read in all of the .result files in a directory (or in a list) and calculate
 * the 2x2 correlation coefficient
 */

class SequenceMutant {
	char[] wildtype;

	/**
	 * Read in all .result files in a directory that also appear in listName
	 * @param mutName
	 * @param wtFile
	 * @param offset
	 */
	public SequenceMutant(String mutName, String wtFile, int offset){
		try{
			//Read in the WT sequence
			FileReader inFile = new FileReader(wtFile);
			BufferedReader in = new BufferedReader(inFile);
			
			String line = in.readLine();
			line = line.toUpperCase();
			wildtype = line.toCharArray();
			in.close();
			
			//Parse the mutant name
			mutName = mutName.toUpperCase();
			StringTokenizer tok = new StringTokenizer(mutName,"_");
			
			while (tok.hasMoreTokens()){
				String curMut = tok.nextToken();
				char wtAA = curMut.charAt(0);
				int aaNum = Integer.parseInt(curMut.substring(1,curMut.length()-1));
				char mutAA = curMut.charAt(curMut.length()-1);
				mut1aa(wtAA,aaNum-offset,mutAA);
			}
			
			System.out.println(wildtype);		

		}catch (Exception e){
			System.err.println(e.getMessage());
			//e.printStackTrace();
		}
	}
	
	/**
	 * Mutate a single residue
	 * @param wtAA
	 * @param aaNum
	 * @param mutAA
	 * @return false if the mutants don't match
	 */
	private boolean mut1aa (char wtAA, int aaNum, char mutAA){
		if (wtAA != wildtype[aaNum]){
			System.err.println(wildtype[aaNum] + " != " + wtAA);
			return false; //The mutants don't match
		} else {
			wildtype[aaNum]=mutAA;			
			return true;
		}
	}

	
	
	/**
	 * Command line
	 * 	Create the sequence for a mutant of 1tsr_b.
	 *	SequenceMutant <mutName> <template> <offset>
	 *	Ex: SequenceMutant r273l_n235k 1tsr_b.fasta 96
	 *	  
	 *
	 * @param args
	 */
	public static void main (String args []) {

		if (args.length != 3){
			System.out.println("Create the sequence for a mutant of 1tsr_b." );
			System.out.println("SequenceMutant <mutName> <template> <offset>" );
			System.out.println("Ex: SequenceMutant r273l_n235k 1tsr_b.fasta 96" );
		} else {
			SequenceMutant mySM = null;
			mySM = new SequenceMutant(args[0],args[1],Integer.parseInt(args[2]));
		}
	}
}