import java.io.*;
import java.util.*;

/**
 *
 * @author Sam Danziger
 *
 * Read in the weka output and convert it to a .result file
 */
class Wekaout2Result {
	public Wekaout2Result(String fileName, boolean useLast){
		try {
			FileReader inFile = new FileReader(fileName);
			BufferedReader in = new BufferedReader(inFile);

			//Pass over the first if useLast == true
			if (useLast == true){
				String line = in.readLine();
				while (!(line.toLowerCase()).startsWith("=== error on test data ===")){
					line = in.readLine();								
				}
			}
			

			//Find confusion matrix
			String line = in.readLine();
			while (!(line.toLowerCase()).startsWith("=== confusion matrix ===")){
				line = in.readLine();								
			}

			//Find part with true value
			while (line.indexOf("0")==-1){
				line = in.readLine();
			}
				
			
			//Read in confusion Matrix
			StringTokenizer tok = new StringTokenizer(line);
			String curTok = tok.nextToken();
			int trueIdx=0; //Index into confusion matrix 1,2;3,4
			if (line.indexOf("1")!=-1){
				if (curTok.equalsIgnoreCase("1")){ 
					trueIdx = 1;
				} else { 
					trueIdx = 2;
				}
			}
			
			
			boolean aActive; //True if a is active
			boolean bActive; //True if b is active
			if ((line.toLowerCase()).indexOf("inactive")!=-1) {
				aActive = false;
				bActive = true;
			} else {
			    aActive = true;
			    bActive = false;	
			}
			
			line = in.readLine();
			tok = new StringTokenizer(line);
			curTok = tok.nextToken();
			if (trueIdx == 0) {
				if (curTok.equalsIgnoreCase("1")){ 
					trueIdx = 3;
				} else { 
					trueIdx = 4;
				}
			}
			in.close();
			
			//Write the .result file
			String pred;
			String truth;
			if (aActive == true) {
				if (trueIdx == 1){
					pred = "active";
					truth = "active";
				} else if (trueIdx == 2){
					pred = "inactive";
					truth = "active";
				} else if (trueIdx == 3){
					pred = "active";
					truth = "inactive";
				} else { //if (trueIdx == 3)
					pred = "inactive";
					truth = "inactive";
				}
			} else { //b is active 
				if (trueIdx == 1){
					pred = "inactive";
					truth = "inactive";
				} else if (trueIdx == 2){
					pred = "active";
					truth = "inactive";
				} else if (trueIdx == 3){
					pred = "inactive";
					truth = "active";
				} else { //if (trueIdx == 3)
					pred = "active";
					truth = "active";
				}
			}
			String outName = getMutName(fileName);
			writeResult(outName,pred,truth);
			
		} catch (Exception e){
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	* Given the mutants Name, and it's true activity, 
	* write the results file
	* @param mutName
	* @param pred
	* @param truth
	*/
   private void writeResult (String mutName, String pred, String truth){
	   try {
		   //String outName = mutName+".result";
		   //PrintWriter out = new PrintWriter( new FileWriter(outName));
		   //out.println("PRED:\t"+pred);
		   //out.println("TRUTH:\t"+truth);
		   //out.close();
		   System.out.println("PRED:\t"+pred);
		   System.out.println("TRUTH:\t"+truth);
	   } catch (Exception e){
		   System.out.println(e.getMessage());
		   e.printStackTrace();			
	   }
		
   }
   
   /**
    * Strip the extension off the fileName
    * @param fileName
    * @return
    */
   private String getMutName(String fileName){
   		int idx = fileName.indexOf(".");
   		String name = fileName.substring(0,idx);
   		return name;
   }
   
   
	/**
	 * Return <dir>/prefix.<fileName>
	 * @param fileName
	 * @param prefix
	 * @return
	 */
	private String addPrefix (String fileName, String prefix){
		String newName = null;

		int idx = fileName.lastIndexOf("/");
		if (idx == -1){
			newName = prefix+"."+fileName;
		} else {
			newName = fileName.subSequence(0,idx+1)+prefix+"."+fileName.subSequence(idx+1,fileName.length());
		}
		return newName;
	}

	/**
	 * Read in the weka output and convert it to a .result file
	 * Wekaout2Result <Wekaout.txt> <useLast - optional>
	 * Wekaout2Result Wekaout.txt
	 *
	 * @param args
	 */
	public static void main (String args []) {
		if (args.length != 2 && args.length != 1){
			System.out.println("Read in the weka output and convert it to a .result file");
			System.out.println("Wekaout2Result <Wekaout.txt> <useLast - optional>" );
			System.out.println("Wekaout2Result Wekaout.txt" );
		} else {
			String arffName = args[0];
			boolean useLast = false;
			if (args.length == 2){
				useLast = true;
			}
			Wekaout2Result myWekaout2Result = new Wekaout2Result(arffName, useLast);
		}
	}

}