import java.io.*;
import java.util.StringTokenizer;

/**
 * 
 * @author Sam Danziger
 *
 *  Read in the output from a cross validated weka run and convert it to a .cc file format
 * 
 * Write results to standard out.
 */

class Wekaout2cc {
	public Wekaout2cc(String fileName, String featureName){
		try {
			FileReader inFile = new FileReader(fileName);
			BufferedReader in = new BufferedReader(inFile);			

			//Strip off leading information			
			String line = in.readLine();
			while (((line.toLowerCase()).indexOf("=== stratified cross-validation ==="))==-1){
				line = in.readLine();
			}

			//Parse for the information
			double accuracy = 0;
			while (!line.startsWith("Correctly Classified Instances ")){
				line = in.readLine();
			}
			StringTokenizer tok = new StringTokenizer(line);
			String tmp = tok.nextToken();
			tmp = tok.nextToken();
			tmp = tok.nextToken();
			tmp = tok.nextToken();
			tmp = tok.nextToken();
			accuracy = (Double.parseDouble(tmp))/100;			

			double kappa = 0;
			while (!line.startsWith("Kappa statistic")){
				line = in.readLine();
			}
			tok = new StringTokenizer(line);
			tmp = tok.nextToken();
			tmp = tok.nextToken();
			tmp = tok.nextToken();
			kappa = Double.parseDouble(tmp);
			
			int tp = 0;
			int fp = 0;
			int fn = 0;
			int tn = 0;
			while (!line.startsWith("=== Confusion Matrix ===")){
				line = in.readLine();
			}
			line = in.readLine();
			line = in.readLine();
			line = in.readLine();
			tok = new StringTokenizer(line);
			tmp = tok.nextToken();
			tp = Integer.parseInt(tmp);
			tmp = tok.nextToken();
			fn = Integer.parseInt(tmp);
			
			line = in.readLine();
			tok = new StringTokenizer(line);
			tmp = tok.nextToken();
			fp = Integer.parseInt(tmp);
			tmp = tok.nextToken();
			tn = Integer.parseInt(tmp);
			
			//  Print the results.  NOTE pred:active, true:active means true positive
			System.out.println("#"+featureName+"\t"+tp+"\t"+fp+"\t"+fn+"\t"+tn);
			System.out.println("tp = "+tp);
			System.out.println("fp = "+fp);
			System.out.println("fn = "+fn);
			System.out.println("tn = "+tn);
			System.out.println("Accuracy = "+accuracy+" = "+accuracy*100+" %");
			System.out.println("Kappa Correlation Coefficient = "+kappa);
			
			
			in.close();
		} catch (Exception e){
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	 * Read in the output from a cross validated weka run and convert it to a .cc file format
	 * 
	 * Write results to standard out.
	 * Wekaout2cc <fileName>
	 * Wekaout2cc wekaout.txt > 1d.2d.cc
	 *
	 * @param args
	 */
	public static void main (String args []) {
		if (args.length != 2){
			System.out.println("Add the name field as the first attribute in a .arff files.  Assumes the convention %name in the line before each data line.");
			System.out.println("Wekaout2cc <fileName> <featureName>" );
			System.out.println("Wekaout2cc wekaout.txt 1d.2d > 1d.2d.cc" );
		} else {
			String fileName = args[0];
			String featureName = args[1];
			Wekaout2cc myWekaout2cc = new Wekaout2cc(fileName, featureName);
			//System.out.println("Completed Execution: AddNameToArff "+arffName);
		}
	}

}