#include "jni.h"

#ifdef _WIN32
#define PATH_SEPARATOR ';'
#else /* UNIX */
#define PATH_SEPARATOR ':'
#endif

#define USER_CLASSPATH ".:../weka.jar"
#define CLASSIFIER "weka.classifiers.m5.M5Prime"
#define MODEL "M5.gz"

typedef struct {
    long cls;
} CWeka_Model;


int            CWeka_Init(char *user_classpath, char *classifier, char *model);
void           CWeka_Finalize();
double         CWeka_Classify(double *d, int len);
CWeka_Model    CWeka_LoadModel(char *classifier, char *model);
void           CWeka_UnloadModel(CWeka_Model *);

JNIEnv *env;
JavaVM *jvm;
jclass cWeka;

main() {
    double d[3];
    int i;

    CWeka_Init(USER_CLASSPATH, CLASSIFIER, MODEL);

    for (i=0; i<10; i++) {
        d[0]=i;
        d[0]=i;
        d[2]=CWeka_Classify(d,1);
        printf("%f\n", d[2]);
    }

    CWeka_Finalize();
}


jcharArray CharToJChar(JNIEnv *env, char *istr){
    int i, len;
    jchar *str ;
    jcharArray jca;

    len=strlen(istr);
    jca = (*env)->NewCharArray(env, len);
    str = (*env)->GetCharArrayElements(env, jca, JNI_FALSE);
    for (i=0;i<len;i++) 
	str[i]=(jchar)istr[i];
    (*env)->ReleaseCharArrayElements(env, jca, str, 0);

    return jca;
}

CWeka_Model CWeka_LoadModel(char *classifier, char *model) {
    CWeka_Model cwm;
    jmethodID mid;
    jcharArray jclassifier, jmodel;

    jclassifier = CharToJChar(env, classifier);
    jmodel      = CharToJChar(env, model);
    mid    =      (*env)->GetStaticMethodID(env, cWeka, "loadModel", "([C[C)Lweka/classifiers/Classifier;");
    cwm.cls=(long)(*env)->CallStaticObjectMethod(env, cWeka, mid, jclassifier, jmodel);

    return cwm;
}

double CWeka_Classify(double *d, int len) {
    static jdoubleArray jda=0;
    static jdoubleArray gjda=0;
    static jmethodID mid=0;
    static double *da;
    static int i;
    static int currentlen=0;
 
    if (gjda!=0 && currentlen<len) {
	(*env)->DeleteGlobalRef(env, gjda);
	gjda=0;
    }

    if (gjda==0) { 
        jda = (*env)->NewDoubleArray(env, len);
        gjda = (*env)->NewGlobalRef(env, jda);
        (*env)->DeleteLocalRef(env, jda);
	currentlen=len;
    }

    if (mid==0) {
        mid=(*env)->GetStaticMethodID(env, cWeka, "classify", "([D)D");
    }

    da =  (*env)->GetPrimitiveArrayCritical(env, gjda, JNI_FALSE);
    bcopy(d, da, sizeof(double)*len);
    (*env)->ReleasePrimitiveArrayCritical(env, gjda, da, 0);

    return (double)(*env)->CallStaticDoubleMethod(env, cWeka, mid, gjda);   
}

int CWeka_Init(char *user_classpath, char *classifier, char *model) {
    JavaVMInitArgs args;
    JavaVMOption options[1];
    jint res;
    char classpath[1024];
    jcharArray jclassifier, jmodel;
    jmethodID mid;
 
    
    args.version = JNI_VERSION_1_2;

   	args.version = JNI_VERSION_1_2;
       	args.nOptions = 1;
	options[0].optionString=(char*)malloc(300*sizeof(char));
	sprintf(options[0].optionString, "-Djava.class.path=%s", user_classpath);
	args.options = options;
	args.ignoreUnrecognized = JNI_FALSE;
   
    /* Create the Java VM */
    res = JNI_CreateJavaVM(&jvm,(void**)&env,&args);
    if (res < 0) {
        fprintf(stderr, "Can't create Java VM.\n");
        return 0;
    }

    cWeka = (*env)->FindClass(env, "CWeka");
    if (cWeka == 0) {
        fprintf(stderr, "Can't find CWeka.\n");
        return 0;
    }

    jmodel      = CharToJChar(env, model);
    jclassifier = CharToJChar(env, classifier);
    mid=(*env)->GetStaticMethodID(env, cWeka, "initModel", "([C[C)V");
    (*env)->CallStaticVoidMethod(env, cWeka, mid, jclassifier, jmodel);

    free(options[0].optionString);
    return 1;
}

void CWeka_Finalize() {
    (*jvm)->DestroyJavaVM(jvm);
}


